MODULE trcbio_medusa
   !!======================================================================
   !!                         ***  MODULE trcbio  ***
   !! TOP :   MEDUSA
   !!======================================================================
   !! History :
   !!  -   !  1999-07  (M. Levy)              original code
   !!  -   !  2000-12  (E. Kestenare)         assign parameters to name individual tracers
   !!  -   !  2001-03  (M. Levy)              LNO3 + dia2d 
   !! 2.0  !  2007-12  (C. Deltel, G. Madec)  F90
   !!  -   !  2008-08  (K. Popova)            adaptation for MEDUSA
   !!  -   !  2008-11  (A. Yool)              continuing adaptation for MEDUSA
   !!  -   !  2010-03  (A. Yool)              updated for branch inclusion
   !!  -   !  2011-08  (A. Yool)              updated for ROAM (see below)
   !!  -   !  2013-10  (A. Yool)              MEDUSA-2 model description
   !!----------------------------------------------------------------------
   !!
#if defined key_roam
   !!----------------------------------------------------------------------
   !! Updates for the ROAM project include:
   !!   - addition of DIC, alkalinity, detrital carbon and oxygen tracers
   !!   - addition of air-sea fluxes of CO2 and oxygen
   !!   - periodic (monthly) calculation of full 3D carbonate chemistry 
   !!   - detrital C:N ratio now free to evolve dynamically
   !!	- benthic storage pools
   !! 
   !! Opportunity also taken to add functionality:
   !!   - switch for Liebig Law (= most-limiting) nutrient uptake
   !!   - switch for accelerated seafloor detritus remineralisation
   !!   - switch for fast -> slow detritus transfer at seafloor
   !!   - switch for ballast vs. Martin vs. Henson fast detritus remin.
   !!   - per GMD referee remarks, xfdfrac3 introduced for grazed PDS
   !!
   !! For future reference, MEDUSA-2 description paper utilises the run
   !! known locally as EXP550; this ran for the period 1860-2099 under
   !! the RCP8.5 scenario; a series of sibling runs using different
   !! scenarios and/or model assumptions exists
   !!----------------------------------------------------------------------
#endif
   !!
#if defined key_medusa
   !!----------------------------------------------------------------------
   !!                                        MEDUSA bio-model
   !!----------------------------------------------------------------------
   !!   trc_bio_medusa        :  
   !!----------------------------------------------------------------------
      USE oce_trc
      USE trc
      USE sms_medusa
      USE lbclnk
      USE prtctl_trc      ! Print control for debugging
      USE trcsed_medusa
      USE sbc_oce         ! surface forcing
      USE sbcrnf          ! surface boundary condition: runoff variables
#if defined key_iomput
      USE iom
#endif
#if defined key_roam
      USE trcco2_medusa
      USE trcoxy_medusa
#endif
      !! AXY (18/01/12): brought in for benthic timestepping
      USE trctrp_lec
      USE trdmld_trc
      USE trdmld_trc_oce

      IMPLICIT NONE
      PRIVATE
      
      PUBLIC   trc_bio_medusa    ! called in ???

   !!* Substitution
#  include "domzgr_substitute.h90"
   !!----------------------------------------------------------------------
   !! NEMO/TOP 2.0 , LOCEAN-IPSL (2007) 
   !! $Id: trcbio.F90 1146 2008-06-25 11:42:56Z rblod $ 
   !! Software governed by the CeCILL licence (modipsl/doc/NEMO_CeCILL.txt)
   !!----------------------------------------------------------------------

CONTAINS

   SUBROUTINE trc_bio_medusa( kt )
      !!---------------------------------------------------------------------
      !!                     ***  ROUTINE trc_bio  ***
      !!
      !! ** Purpose :   compute the now trend due to biogeochemical processes
      !!              and add it to the general trend of passive tracers equations
      !!
      !! ** Method  :   each now biological flux is calculated in function of now
      !!              concentrations of tracers.
      !!              depending on the tracer, these fluxes are sources or sinks.
      !!              the total of the sources and sinks for each tracer
      !!              is added to the general trend.
      !!        
      !!                      tra = tra + zf...tra - zftra...
      !!                                     |         |
      !!                                     |         |
      !!                                  source      sink
      !!        
      !!              IF 'key_trc_diabio' defined , the biogeochemical trends
      !!              for passive tracers are saved for futher diagnostics.
      !!---------------------------------------------------------------------
      !!
      !!----------------------------------------------------------------------            
      !! Variable conventions
      !!----------------------------------------------------------------------
      !!
      !! names: z*** - state variable 
      !!        f*** - function (or temporary variable used in part of a function)
      !!        x*** - parameter
      !!        b*** - right-hand part (sources and sinks)
      !!        i*** - integer variable (usually used in yes/no flags)
      !!
      !! time (integer timestep)
      INTEGER, INTENT( in ) ::    kt
      !!
      !! spatial array indices
      INTEGER  ::    ji,jj,jk,jn
      !!
      !! AXY (27/07/10): add in indices for depth horizons (for sinking flux
      !!                 and seafloor iron inputs)
      !! INTEGER  ::    i0100, i0200, i0500, i1000, i1100
      !!
      !! model state variables
      REAL(wp) ::    zchn,zchd,zphn,zphd,zpds,zzmi
      REAL(wp) ::    zzme,zdet,zdtc,zdin,zsil,zfer
      REAL(wp) ::    zage
#if defined key_roam
      REAL(wp) ::    zdic, zalk, zoxy
      REAL(wp) ::    ztmp, zsal
#endif
      !!
      !! integrated source and sink terms
      REAL(wp) ::    b0
      REAL(wp) ::    bchn,bchd,bphn,bphd,bpds,bzmi
      REAL(wp) ::    bzme,bdet,bdtc,bdin,bsil,bfer
#if defined key_roam
      REAL(wp) ::    bdic, balk, boxy
#endif
      !!
      !! primary production and chl related quantities      
      REAL(wp) ::    fthetan,faln,fchn1,fchn,fjln,fprn,frn
      REAL(wp) ::    fthetad,fald,fchd1,fchd,fjld,fprd,frd
      !! AXY (03/02/11): add in Liebig terms
      REAL(wp) ::    fpnlim, fpdlim
      !! AXY (16/07/09): add in Eppley curve functionality
      REAL(wp) ::    loc_T,fun_T,xvpnT,xvpdT
      INTEGER  ::    ieppley
      !! AXY (01/03/10): add in mixed layer PP diagnostics
      REAL(wp) ::    fprn_ml,fprd_ml
      !!
      !! nutrient limiting factors
      REAL(wp) ::    fnln,ffln            !! N and Fe
      REAL(wp) ::    fnld,ffld,fsld,fsld2 !! N, Fe and Si
      !!
      !! silicon cycle
      REAL(wp) ::    fsin,fnsi,fsin1,fnsi1,fnsi2,fprds,fsdiss
      !!
      !! iron cycle; includes parameters for Parekh et al. (2005) iron scheme
      REAL(wp) ::    ffetop,ffebot,ffescav
      REAL(wp) ::    xLgF, xFeT, xFeF, xFeL, xFree  !! state variables for iron-ligand system
      REAL(wp) ::    xb_coef_tmp, xb2M4ac           !! iron-ligand parameters
      REAL(wp) ::    xmaxFeF,fdeltaFe               !! max Fe' parameters
      !!
      !! local parameters for Moore et al. (2004) alternative scavenging scheme
      REAL(wp) ::    fbase_scav,fscal_sink,fscal_part,fscal_scav
      !!
      !! local parameters for Moore et al. (2008) alternative scavenging scheme
      REAL(wp) ::    fscal_csink,fscal_sisink,fscal_casink
      !!
      !! local parameters for Galbraith et al. (2010) alternative scavenging scheme
      REAL(wp) ::    xCscav1, xCscav2, xk_org, xORGscav  !! organic portion of scavenging
      REAL(wp) ::    xk_inorg, xINORGscav                !! inorganic portion of scavenging
      !!
      !! microzooplankton grazing
      REAL(wp) ::    fmi1,fmi,fgmipn,fgmid,fgmidc
      REAL(wp) ::    finmi,ficmi,fstarmi,fmith,fmigrow,fmiexcr,fmiresp
      !!
      !! mesozooplankton grazing
      REAL(wp) ::    fme1,fme,fgmepn,fgmepd,fgmepds,fgmezmi,fgmed,fgmedc
      REAL(wp) ::    finme,ficme,fstarme,fmeth,fmegrow,fmeexcr,fmeresp
      !!
      !! mortality/Remineralisation (defunct parameter "fz" removed)
      REAL(wp) ::    fdpn,fdpd,fdpds,fdzmi,fdzme,fdd
#if defined key_roam
      REAL(wp) ::    fddc
#endif
      REAL(wp) ::    fdpn2,fdpd2,fdpds2,fdzmi2,fdzme2
      REAL(wp) ::    fslown, fslownflux
      REAL(wp) ::    fslowc, fslowcflux
      REAL(wp) ::    fregen,fregensi
      REAL(wp), DIMENSION(jpi,jpj) ::    fregenfast,fregenfastsi
#if defined key_roam
      REAL(wp) ::    fregenc
      REAL(wp), DIMENSION(jpi,jpj) ::    fregenfastc
#endif
      !!
      !! particle flux
      REAL(WP) ::    fthk,fdep,fdep1,fdep2,flat,fcaco3
      REAL(WP) ::    ftempn,ftempsi,ftempfe,ftempc,ftempca
      REAL(wp) ::    freminn,freminsi,freminfe,freminc,freminca
      REAL(wp), DIMENSION(jpi,jpj) ::    ffastn,ffastsi,ffastfe,ffastc,ffastca
      REAL(wp) ::    fleftn,fleftsi,fleftfe,fleftc,fleftca
      REAL(wp) ::    fheren,fheresi,fherefe,fherec,fhereca
      REAL(wp) ::    fprotf
      REAL(wp) ::    fsedn,fsedsi,fsedfe,fsedc,fsedca
      REAL(wp), DIMENSION(jpi,jpj) ::    fccd
      REAL(wp) ::    fccd_dep
      !!
      !! AXY (06/07/11): alternative fast detritus schemes
      REAL(wp) ::    fb_val, fl_sst
      !!
      !! AXY (08/07/11): fate of fast detritus reaching the seafloor
      REAL(wp) ::    ffast2slown,ffast2slowfe,ffast2slowc
      !!
      !! conservation law
      REAL(wp) ::    fnit0,fsil0,ffer0 
#if defined key_roam
      REAL(wp) ::    fcar0,falk0,foxy0 
#endif      
      !! 
      !! temporary variables
      REAL(wp) ::    fq0,fq1,fq2,fq3,fq4,fq5,fq6,fq7,fq8,fq9
      !!
      !! water column nutrient and flux integrals
      REAL(wp) ::    ftot_n,ftot_si,ftot_fe
      REAL(wp), DIMENSION(jpi,jpj) ::    fflx_n,fflx_si,fflx_fe
      REAL(wp), DIMENSION(jpi,jpj) ::    fifd_n,fifd_si,fifd_fe
      REAL(wp), DIMENSION(jpi,jpj) ::    fofd_n,fofd_si,fofd_fe
#if defined key_roam
      REAL(wp) ::    ftot_c,ftot_a,ftot_o2
      REAL(wp), DIMENSION(jpi,jpj) ::    fflx_c,fflx_a,fflx_o2
      REAL(wp), DIMENSION(jpi,jpj) ::    fifd_c,fifd_a,fifd_o2
      REAL(wp), DIMENSION(jpi,jpj) ::    fofd_c,fofd_a,fofd_o2
#endif
      !!
      !! zooplankton grazing integrals
      REAL(wp), DIMENSION(jpi,jpj) ::    fzmi_i,fzmi_o,fzme_i,fzme_o
      !!
      !! limitation term temporary variables
      REAL(wp), DIMENSION(jpi,jpj) ::    ftot_pn,ftot_pd
      !! use ballast scheme (1) or simple exponential scheme (0; a conservation test)
      INTEGER  ::    iball
      !! use biological fluxes (1) or not (0)
      INTEGER  ::    ibio_switch
      !!
      !! diagnose fluxes (should only be used in 1D runs)
      INTEGER  ::    idf, idfval
      !!
      !! nitrogen and silicon production and consumption
      REAL(wp) ::    fn_prod, fn_cons, fs_prod, fs_cons
      REAL(wp), DIMENSION(jpi,jpj) ::    fnit_prod, fnit_cons, fsil_prod, fsil_cons
#if defined key_roam
      !!
      !! flags to help with calculating the position of the CCD
      INTEGER, DIMENSION(jpi,jpj) ::     i2_omcal,i2_omarg
      !!
      !! ROAM air-sea flux and diagnostic parameters
      REAL(wp) ::    f_wind, f_pco2a
      REAL(wp) ::    f_ph, f_pco2w, f_h2co3, f_hco3, f_co3, f_co2flux
      REAL(wp) ::    f_TDIC, f_TALK, f_dcf, f_henry
      REAL(wp) ::    f_uwind, f_vwind, f_pp0
      REAL(wp) ::    f_kw660, f_o2flux, f_o2sat
      REAL(wp), DIMENSION(jpi,jpj) ::    f_omcal, f_omarg
      REAL(wp) ::    f_year
      INTEGER  ::    i_year
      INTEGER  ::    iyr1, iyr2
      !!
      !! carbon, alkalinity production and consumption
      REAL(wp) ::    fc_prod, fc_cons, fa_prod, fa_cons
      REAL(wp), DIMENSION(jpi,jpj) ::    fcar_prod, fcar_cons
      !!
      !! oxygen production and consumption (and non-consumption)
      REAL(wp) ::    fo2_prod, fo2_cons, fo2_ncons, fo2_ccons
      REAL(wp), DIMENSION(jpi,jpj) ::    foxy_prod, foxy_cons, foxy_anox
#endif
      !! 
      !! benthic fluxes
      INTEGER  ::    ibenthic
      REAL(wp), DIMENSION(jpi,jpj) :: f_sbenin_n, f_sbenin_fe,              f_sbenin_c
      REAL(wp), DIMENSION(jpi,jpj) :: f_fbenin_n, f_fbenin_fe, f_fbenin_si, f_fbenin_c, f_fbenin_ca
      REAL(wp), DIMENSION(jpi,jpj) :: f_benout_n, f_benout_fe, f_benout_si, f_benout_c, f_benout_ca
      REAL(wp) ::    zfact
      !! 
      !! benthic fluxes of CaCO3 that shouldn't happen because of lysocline
      REAL(wp), DIMENSION(jpi,jpj) :: f_benout_lyso_ca
      !!
      !! riverine fluxes
      REAL(wp), DIMENSION(jpi,jpj) :: f_runoff, f_riv_n, f_riv_si, f_riv_c, f_riv_alk
      !! AXY (19/07/12): variables for local riverine fluxes to handle inputs below surface
      REAL(wp) ::    f_riv_loc_n, f_riv_loc_si, f_riv_loc_c, f_riv_loc_alk

#if defined key_trc_diaadd
      !! local array to store grid cell thickness (for 2D diagnostics)
      REAL(wp), DIMENSION(jpi,jpj,jpk) ::   ze3t
#endif

      !!---------------------------------------------------------------------

      IF( kt == nit000 ) THEN
         IF(lwp) WRITE(numout,*)
         IF(lwp) WRITE(numout,*) ' trc_bio: MEDUSA bio-model'
         IF(lwp) WRITE(numout,*) ' ~~~~~~~'
      ENDIF

      !! AXY (13/01/12): is benthic model properly interactive? 0 = no, 1 = yes
      ibenthic = 1

      !! not sure what this is for; it's not used anywhere; commenting out
      !! fbodn(:,:) = 0.e0   

#if defined key_trc_diaadd
      !! setup up arrays for 2D diagnostics if these are included
      !!
      !! create array to convert volumetric fluxes to per metre squared per day
      ze3t(:,:,:) = 0.e0
      DO jk = 1,jpk
         !! AXY (04/03/09): fluxes are already per day; no need to correct for seconds
         !! ze3t(:,:,jk) = fse3t(:,:,jk) * 86400.
         ze3t(:,:,jk) = fse3t(:,:,jk)
      END DO 
      !!
      !! blank 2D diagnostic array
      trc2d(:,:,:) = 0.e0
      !!
      !! blank 3D diagnostic array
      trc3d(:,:,:,:) = 0.e0
#endif

      !!----------------------------------------------------------------------
      !! b0 is present for debugging purposes; using b0 = 0 sets the tendency
      !! terms of all biological equations to 0.
      !!----------------------------------------------------------------------
      !!
      b0 = 1.

      !!----------------------------------------------------------------------
      !! fast detritus ballast scheme (0 = no; 1 = yes)
      !! alternative to ballast scheme is same scheme but with no ballast
      !! protection (not dissimilar to Martin et al., 1987)
      !!----------------------------------------------------------------------
      !!
      iball = 1

      !!----------------------------------------------------------------------
      !! full flux diagnostics (0 = no; 1 = yes); appear in ocean.output
      !! these should *only* be used in 1D since they give comprehensive
      !! output for ecological functions in the model; primarily used in
      !! debugging
      !!----------------------------------------------------------------------
      !!
      idf    = 0
      !!
      !! timer mechanism
      if (kt/120*120.eq.kt) then
         idfval = 1
      else
         idfval = 0
      endif

      !!----------------------------------------------------------------------
      !! blank fast-sinking detritus 2D fields
      !!----------------------------------------------------------------------
      !!
      ffastn(:,:)  = 0.0        !! organic nitrogen
      ffastsi(:,:) = 0.0        !! biogenic silicon
      ffastfe(:,:) = 0.0        !! organic iron
      ffastc(:,:)  = 0.0        !! organic carbon
      ffastca(:,:) = 0.0        !! biogenic calcium carbonate
      !!
      fregenfast(:,:)   = 0.0   !! integrated  N regeneration (fast detritus)
      fregenfastsi(:,:) = 0.0   !! integrated Si regeneration (fast detritus)
      !!
      fccd(:,:)    = 0.0        !! last depth level before CCD

      !!----------------------------------------------------------------------
      !! blank nutrient/flux inventories
      !!----------------------------------------------------------------------
      !!
      fflx_n(:,:)  = 0.0        !! nitrogen flux total
      fflx_si(:,:) = 0.0        !! silicon  flux total
      fflx_fe(:,:) = 0.0        !! iron     flux total
      fifd_n(:,:)  = 0.0        !! nitrogen fast detritus production
      fifd_si(:,:) = 0.0        !! silicon  fast detritus production
      fifd_fe(:,:) = 0.0        !! iron     fast detritus production
      fofd_n(:,:)  = 0.0        !! nitrogen fast detritus remineralisation
      fofd_si(:,:) = 0.0        !! silicon  fast detritus remineralisation
      fofd_fe(:,:) = 0.0        !! iron     fast detritus remineralisation
#if defined key_roam
      fflx_c(:,:)  = 0.0        !! carbon     flux total
      fflx_a(:,:)  = 0.0        !! alkalinity flux total
      fflx_o2(:,:) = 0.0        !! oxygen     flux total
      fifd_c(:,:)  = 0.0        !! carbon     fast detritus production
      fifd_a(:,:)  = 0.0        !! alkalinity fast detritus production
      fifd_o2(:,:) = 0.0        !! oxygen     fast detritus production
      fofd_c(:,:)  = 0.0        !! carbon     fast detritus remineralisation
      fofd_a(:,:)  = 0.0        !! alkalinity fast detritus remineralisation
      fofd_o2(:,:) = 0.0        !! oxygen     fast detritus remineralisation
      !!
      fnit_prod(:,:) = 0.0      !! (organic) nitrogen production
      fnit_cons(:,:) = 0.0      !! (organic) nitrogen consumption
      fcar_prod(:,:) = 0.0      !! (organic) carbon production
      fcar_cons(:,:) = 0.0      !! (organic) carbon consumption
      !!
      foxy_prod(:,:) = 0.0      !! oxygen production
      foxy_cons(:,:) = 0.0      !! oxygen consumption
      foxy_anox(:,:) = 0.0      !! unrealised oxygen consumption
#endif
      ftot_pn(:,:) = 0.0        !! integrated non-diatom phytoplankton
      ftot_pd(:,:) = 0.0        !! integrated diatom     phytoplankton
      !!
      fzmi_i(:,:)  = 0.0        !! material grazed by microzooplankton
      fzmi_o(:,:)  = 0.0        !! ... sum of fate of this material
      fzme_i(:,:)  = 0.0        !! material grazed by  mesozooplankton
      fzme_o(:,:)  = 0.0        !! ... sum of fate of this material
      !!
      f_sbenin_n(:,:)  = 0.0    !! slow detritus N  -> benthic pool
      f_sbenin_fe(:,:) = 0.0    !! slow detritus Fe -> benthic pool
      f_sbenin_c(:,:)  = 0.0    !! slow detritus C  -> benthic pool
      f_fbenin_n(:,:)  = 0.0    !! fast detritus N  -> benthic pool
      f_fbenin_fe(:,:) = 0.0    !! fast detritus Fe -> benthic pool
      f_fbenin_si(:,:) = 0.0    !! fast detritus Si -> benthic pool
      f_fbenin_c(:,:)  = 0.0    !! fast detritus C  -> benthic pool
      f_fbenin_ca(:,:) = 0.0    !! fast detritus Ca -> benthic pool
      !!
      f_benout_n(:,:)  = 0.0    !! benthic N  pool  -> dissolved
      f_benout_fe(:,:) = 0.0    !! benthic Fe pool  -> dissolved
      f_benout_si(:,:) = 0.0    !! benthic Si pool  -> dissolved
      f_benout_c(:,:)  = 0.0    !! benthic C  pool  -> dissolved
      f_benout_ca(:,:) = 0.0    !! benthic Ca pool  -> dissolved
      !!
      f_benout_lyso_ca(:,:) = 0.0 !! benthic Ca pool  -> dissolved (when it shouldn't!)
      !!
      f_runoff(:,:)  = 0.0      !! riverine runoff
      f_riv_n(:,:)   = 0.0      !! riverine N   input 
      f_riv_si(:,:)  = 0.0      !! riverine Si  input 
      f_riv_c(:,:)   = 0.0      !! riverine C   input 
      f_riv_alk(:,:) = 0.0      !! riverine alk input 

      ! DO jn = 1,jptra
      !    if (lwp) write (numout,*) 'NAN-CHECK', jn, minval(trn(:,:,:,jn)), maxval(trn(:,:,:,jn))
      ! ENDDO	

#if defined key_roam
      !!----------------------------------------------------------------------
      !! calculate atmospheric pCO2
      !!----------------------------------------------------------------------
      !!
      !! what's atmospheric pCO2 doing? (data start in 1859)
      iyr1  = nyear - 1859 + 1
      iyr2  = iyr1 + 1
      if (iyr1 .le. 1) then
         !! before 1860
         f_pco2a = hist_pco2(1)
      elseif (iyr2 .ge. 242) then
         !! after 2099
         f_pco2a = hist_pco2(242)
      else
         !! just right
         fq0 = hist_pco2(iyr1)
         fq1 = hist_pco2(iyr2)
         fq2 = real(nsec_day) / (60.0 * 60.0 * 24.0)
	 !! AXY (14/06/12): tweaked to make more sense (and be correct)
	 fq3 = (real(nday_year) - 1.0 + fq2) / 360.0
         fq4 = (fq0 * (1.0 - fq3)) + (fq1 * fq3)
         f_pco2a = fq4
      endif
# if defined key_pi_co2
      f_pco2a = hist_pco2(1)
      IF(lwp) WRITE(numout,*) ' MEDUSA atm pCO2  = FIXED'
# endif     
      IF(lwp) WRITE(numout,*) ' MEDUSA nyear     =', nyear
      IF(lwp) WRITE(numout,*) ' MEDUSA nsec_day  =', real(nsec_day)
      IF(lwp) WRITE(numout,*) ' MEDUSA nday_year =', real(nday_year)
      IF(lwp) WRITE(numout,*) ' MEDUSA fq0       =', fq0
      IF(lwp) WRITE(numout,*) ' MEDUSA fq1       =', fq1
      IF(lwp) WRITE(numout,*) ' MEDUSA fq2       =', fq2
      IF(lwp) WRITE(numout,*) ' MEDUSA fq3       =', fq3
      IF(lwp) WRITE(numout,*) ' MEDUSA atm pCO2  =', f_pco2a

#endif

#if defined key_roam
      IF( kt == nit000 .or. mod(kt,720) == 0 ) THEN
         !!----------------------------------------------------------------------
         !! Calculate the carbonate chemistry for the whole ocean on the first
         !! simulation timestep and every month subsequently; the resulting 3D
         !! field of omega calcite is used to determine the depth of the CCD
         !!----------------------------------------------------------------------
         !!
         !! blank flags
         i2_omcal(:,:) = 0
         i2_omarg(:,:) = 0
         !! loop over 3D space
         DO jk = 1,jpk
            DO jj = 2,jpjm1
               DO ji = 2,jpim1
                  !! OPEN wet point IF..THEN loop
                  if (tmask(ji,jj,jk).eq.1) then
                     !! carbonate chemistry
                     fdep2 = fsdept(ji,jj,jk)           !! set up level midpoint
                     !! set up required state variables
                     zdic = max(0.,trn(ji,jj,jk,jpdic)) !! dissolved inorganic carbon
                     zalk = max(0.,trn(ji,jj,jk,jpalk)) !! alkalinity
                     ztmp = tn(ji,jj,jk)                !! temperature
                     zsal = sn(ji,jj,jk)                !! salinity
                     !! calculate carbonate chemistry at grid cell midpoint
                     CALL trc_co2_medusa( ztmp, zsal, zdic, zalk, fdep2, 5.0, f_pco2a, &    ! inputs
                     f_ph, f_pco2w, f_h2co3, f_hco3, f_co3, f_omcal(ji,jj),            &    ! outputs
                     f_omarg(ji,jj), f_co2flux, f_TDIC, f_TALK, f_dcf, f_henry)             ! outputs
                     !! store 3D outputs
                     f3_pH(ji,jj,jk)    = f_ph
                     f3_h2co3(ji,jj,jk) = f_h2co3
                     f3_hco3(ji,jj,jk)  = f_hco3
                     f3_co3(ji,jj,jk)   = f_co3
                     f3_omcal(ji,jj,jk) = f_omcal(ji,jj)
                     f3_omarg(ji,jj,jk) = f_omarg(ji,jj)
                     !! CCD calculation: calcite
                     if (i2_omcal(ji,jj) .eq. 0 .and. f_omcal(ji,jj) .lt. 1.0) then
                        if (jk .eq. 1) then
                           f2_ccd_cal(ji,jj) = fdep2
                        else
                           fq0 = f3_omcal(ji,jj,jk-1) - f_omcal(ji,jj)
                           fq1 = f3_omcal(ji,jj,jk-1) - 1.0
                           fq2 = fq1 / fq0
                           fq3 = fdep2 - fsdept(ji,jj,jk-1)
                           fq4 = fq2 * fq3
                           f2_ccd_cal(ji,jj) = fsdept(ji,jj,jk-1) + fq4
                        endif
                        i2_omcal(ji,jj)   = 1
                     endif
                     if ( i2_omcal(ji,jj) .eq. 0 .and. jk .eq. (mbathy(ji,jj)-1) ) then
                        !! reached seafloor and still no dissolution; set to seafloor (W-point)
                        f2_ccd_cal(ji,jj) = fsdepw(ji,jj,jk+1)
                        i2_omcal(ji,jj)   = 1
                     endif
                     !! CCD calculation: aragonite
                     if (i2_omarg(ji,jj) .eq. 0 .and. f_omarg(ji,jj) .lt. 1.0) then
                        if (jk .eq. 1) then
                           f2_ccd_arg(ji,jj) = fdep2
                        else
                           fq0 = f3_omarg(ji,jj,jk-1) - f_omarg(ji,jj)
                           fq1 = f3_omarg(ji,jj,jk-1) - 1.0
                           fq2 = fq1 / fq0
                           fq3 = fdep2 - fsdept(ji,jj,jk-1)
                           fq4 = fq2 * fq3
                           f2_ccd_arg(ji,jj) = fsdept(ji,jj,jk-1) + fq4
                        endif
                        i2_omarg(ji,jj)   = 1
                     endif
                     if ( i2_omarg(ji,jj) .eq. 0 .and. jk .eq. (mbathy(ji,jj)-1) ) then
                        !! reached seafloor and still no dissolution; set to seafloor (W-point)
                        f2_ccd_arg(ji,jj) = fsdepw(ji,jj,jk+1)
                        i2_omarg(ji,jj)   = 1
                     endif
                  endif
               ENDDO
            ENDDO
         ENDDO
      ENDIF
#endif

      !!----------------------------------------------------------------------
      !! MEDUSA has unified equation through the water column
      !! (Diff. from LOBSTER which has two sets: bio- and non-bio layers) 
      !! Statement below in LOBSTER is different: DO jk = 1, jpkbm1          
      !!----------------------------------------------------------------------
      !!
      !! NOTE: the ordering of the loops below differs from that of some other
      !! models; looping over the vertical dimension is the outermost loop and
      !! this complicates some calculations (e.g. storage of vertical fluxes
      !! that can otherwise be done via a singular variable require 2D fields
      !! here); however, these issues are relatively easily resolved, but the
      !! loops CANNOT be reordered without potentially causing code efficiency
      !! problems (e.g. array indexing means that reordering the loops would
      !! require skipping between widely-spaced memory location; potentially
      !! outside those immediately cached)
      !!
      !! OPEN vertical loop
      DO jk = 1,jpk
         !! OPEN horizontal loops
         DO jj = 2,jpjm1
         DO ji = 2,jpim1
            !! OPEN wet point IF..THEN loop
            if (tmask(ji,jj,jk).eq.1) then               

               !!======================================================================
               !! SETUP LOCAL GRID CELL
               !!======================================================================
               !!
               !!---------------------------------------------------------------------
               !! Some notes on grid vertical structure
               !! - fsdepw(ji,jj,jk) is the depth of the upper surface of level jk
               !! - fsde3w(ji,jj,jk) is *approximately* the midpoint of level jk
               !! - fse3t(ji,jj,jk)  is the thickness of level jk
               !!---------------------------------------------------------------------
               !!
               !! AXY (11/12/08): set up level thickness
               fthk  = fse3t(ji,jj,jk)
	       !! AXY (25/02/10): set up level depth (top of level)
	       fdep  = fsdepw(ji,jj,jk)
               !! AXY (01/03/10): set up level depth (bottom of level)
               fdep1 = fdep + fthk
               !!
               !! set up model tracers
               !! negative values of state variables are not allowed to
               !! contribute to the calculated fluxes
               zchn = max(0.,trn(ji,jj,jk,jpchn)) !! non-diatom chlorophyll
               zchd = max(0.,trn(ji,jj,jk,jpchd)) !! diatom chlorophyll
               zphn = max(0.,trn(ji,jj,jk,jpphn)) !! non-diatoms
               zphd = max(0.,trn(ji,jj,jk,jpphd)) !! diatoms
               zpds = max(0.,trn(ji,jj,jk,jppds)) !! diatom silicon
               !! AXY (28/01/10): probably need to take account of chl/biomass connection
	       if (zchn.eq.0.) zphn = 0.
	       if (zchd.eq.0.) zphd = 0.
	       if (zphn.eq.0.) zchn = 0.
	       if (zphd.eq.0.) zchd = 0.
               zzmi = max(0.,trn(ji,jj,jk,jpzmi)) !! microzooplankton
               zzme = max(0.,trn(ji,jj,jk,jpzme)) !! mesozooplankton
               zdet = max(0.,trn(ji,jj,jk,jpdet)) !! detrital nitrogen
               zdin = max(0.,trn(ji,jj,jk,jpdin)) !! dissolved inorganic nitrogen
               zsil = max(0.,trn(ji,jj,jk,jpsil)) !! dissolved silicic acid
               zfer = max(0.,trn(ji,jj,jk,jpfer)) !! dissolved "iron"
#if defined key_roam
               zdtc = max(0.,trn(ji,jj,jk,jpdtc)) !! detrital carbon
               zdic = max(0.,trn(ji,jj,jk,jpdic)) !! dissolved inorganic carbon
               zalk = max(0.,trn(ji,jj,jk,jpalk)) !! alkalinity
               zoxy = max(0.,trn(ji,jj,jk,jpoxy)) !! oxygen
               !!
               !! also need physical parameters for gas exchange calculations
               ztmp = tn(ji,jj,jk)
               zsal = sn(ji,jj,jk)
#else
               zdtc = zdet * xthetad              !! implicit detrital carbon
#endif
               if (idf.eq.1) then
               !! AXY (15/01/10)
               if (trn(ji,jj,jk,jpdin).lt.0.) then
                  IF (lwp) write (numout,*) '------------------------------'
                  IF (lwp) write (numout,*) 'NEGATIVE DIN ERROR =', trn(ji,jj,jk,jpdin)
                  IF (lwp) write (numout,*) 'NEGATIVE DIN ERROR @', ji, jj, jk, kt
               endif
               if (trn(ji,jj,jk,jpsil).lt.0.) then
                  IF (lwp) write (numout,*) '------------------------------'
                  IF (lwp) write (numout,*) 'NEGATIVE SIL ERROR =', trn(ji,jj,jk,jpsil)
                  IF (lwp) write (numout,*) 'NEGATIVE SIL ERROR @', ji, jj, jk, kt
               endif
#if defined key_roam
               if (trn(ji,jj,jk,jpdic).lt.0.) then
                  IF (lwp) write (numout,*) '------------------------------'
                  IF (lwp) write (numout,*) 'NEGATIVE DIC ERROR =', trn(ji,jj,jk,jpdic)
                  IF (lwp) write (numout,*) 'NEGATIVE DIC ERROR @', ji, jj, jk, kt
               endif
               if (trn(ji,jj,jk,jpalk).lt.0.) then
                  IF (lwp) write (numout,*) '------------------------------'
                  IF (lwp) write (numout,*) 'NEGATIVE ALK ERROR =', trn(ji,jj,jk,jpalk)
                  IF (lwp) write (numout,*) 'NEGATIVE ALK ERROR @', ji, jj, jk, kt
               endif
               if (trn(ji,jj,jk,jpoxy).lt.0.) then
                  IF (lwp) write (numout,*) '------------------------------'
                  IF (lwp) write (numout,*) 'NEGATIVE OXY ERROR =', trn(ji,jj,jk,jpoxy)
                  IF (lwp) write (numout,*) 'NEGATIVE OXY ERROR @', ji, jj, jk, kt
               endif
#endif
               endif

               !! report state variable values
               if (idf.eq.1.AND.idfval.eq.1) then
                  IF (lwp) write (numout,*) '------------------------------'
                  IF (lwp) write (numout,*) 'fthk(',jk,') = ', fthk
                  IF (lwp) write (numout,*) 'zphn(',jk,') = ', zphn
                  IF (lwp) write (numout,*) 'zphd(',jk,') = ', zphd
                  IF (lwp) write (numout,*) 'zpds(',jk,') = ', zpds
                  IF (lwp) write (numout,*) 'zzmi(',jk,') = ', zzmi
                  IF (lwp) write (numout,*) 'zzme(',jk,') = ', zzme
                  IF (lwp) write (numout,*) 'zdet(',jk,') = ', zdet
                  IF (lwp) write (numout,*) 'zdin(',jk,') = ', zdin
                  IF (lwp) write (numout,*) 'zsil(',jk,') = ', zsil
                  IF (lwp) write (numout,*) 'zfer(',jk,') = ', zfer
#if defined key_roam
                  IF (lwp) write (numout,*) 'zdtc(',jk,') = ', zdtc
                  IF (lwp) write (numout,*) 'zdic(',jk,') = ', zdic
                  IF (lwp) write (numout,*) 'zalk(',jk,') = ', zalk
                  IF (lwp) write (numout,*) 'zoxy(',jk,') = ', zoxy                  
#endif
               endif

               if (idf.eq.1.AND.idfval.eq.1.AND.jk.eq.1) then
                  IF (lwp) write (numout,*) '------------------------------'
                  IF (lwp) write (numout,*) 'dustmo(1) = ', dustmo(ji,jj,1)
                  IF (lwp) write (numout,*) 'dustmo(2) = ', dustmo(ji,jj,2)
                  IF (lwp) write (numout,*) 'dust      = ', dust(ji,jj)
               endif

               !! sum tracers for inventory checks
               ftot_n  = fthk * ( zphn + zphd + zzmi + zzme + zdet + zdin )
               ftot_si = fthk * ( zpds + zsil )
               ftot_fe = fthk * ( xrfn * ( zphn + zphd + zzmi + zzme + zdet ) + zfer )
#if defined key_roam
               ftot_c  = fthk * ( (xthetapn * zphn) + (xthetapd * zphd) + &
                         (xthetazmi * zzmi) + (xthetazme * zzme) + zdtc + &
                         zdic )
               ftot_a  = fthk * ( zalk )
               ftot_o2 = fthk * ( zoxy )
#endif
               !!======================================================================
               !! LOCAL GRID CELL CALCULATIONS
               !!======================================================================
               !!
#if defined key_roam
               if ( jk .eq. 1 ) then
                  !!----------------------------------------------------------------------
                  !! Air-sea gas exchange
                  !!----------------------------------------------------------------------
                  !!
                  !! a bit of set up ...
                  ! f_uwind = zwnd_i(ji,jj)
                  ! f_vwind = zwnd_j(ji,jj)
                  f_uwind = zwind_i(ji,jj)
                  f_vwind = zwind_j(ji,jj)
                  f_wind  = ((f_uwind**2.0) + (f_vwind**2.0))**0.5
                  f_pp0   = 1.0
                  !! IF(lwp) WRITE(numout,*) ' MEDUSA ztmp    =', ztmp
                  !! IF(lwp) WRITE(numout,*) ' MEDUSA zwind_i =', zwind_i(ji,jj)
                  !! IF(lwp) WRITE(numout,*) ' MEDUSA zwind_j =', zwind_j(ji,jj)
                  !! IF(lwp) WRITE(numout,*) ' MEDUSA f_wind  =', f_wind
                  !! IF(lwp) WRITE(numout,*) ' MEDUSA fr_i    =', fr_i(ji,jj)
                  !!
                  !! carbon dioxide (CO2); Jerry Blackford code (ostensibly OCMIP-2, but not)
                  CALL trc_co2_medusa( ztmp, zsal, zdic, zalk, 0.0, f_wind, f_pco2a, &     ! inputs
                  f_ph, f_pco2w, f_h2co3, f_hco3, f_co3, f_omcal(ji,jj),             &     ! outputs
                  f_omarg(ji,jj), f_co2flux, f_TDIC, f_TALK, f_dcf, f_henry )              ! outputs
                  !!
                  !! already in right units; correct for sea-ice; divide through by layer thickness
                  f_co2flux = (1. - fr_i(ji,jj)) * f_co2flux / fthk
                  !!
                  !! oxygen (O2); OCMIP-2 code
                  CALL trc_oxy_medusa( ztmp, zsal, f_uwind, f_vwind, f_pp0, zoxy / 1000., fthk, &  ! inputs
                  f_kw660, f_o2flux, f_o2sat )                                                     ! outputs
                  !!
                  !! mol/m3/s -> mmol/m3/d; correct for sea-ice
                  f_o2flux  = (1. - fr_i(ji,jj)) * f_o2flux * 1000. * 60. * 60. * 24. 
                  f_o2sat   = f_o2sat  * 1000.
               endif
#endif

               if ( jk .eq. 1 ) then
                  !!----------------------------------------------------------------------
                  !! River inputs
                  !!----------------------------------------------------------------------
                  !!
                  !! runoff comes in as        kg / m2 / s
		  !! used and written out as   m3 / m2 / d (= m / d)
		  !! where                     1000 kg / m2 / d = 1 m3 / m2 / d = 1 m / d
                  f_runoff(ji,jj) = sf_rnf(1)%fnow(ji,jj) / 1000. * 60. * 60. * 24.
                  !!
		  !! nutrients are added via rivers to the model in one of two ways:
                  !!   1. via river concentration; i.e. the average nutrient concentration
                  !!      of a river water is described by a spatial file, and this is
                  !!      multiplied by runoff to give a nutrient flux
                  !!   2. via direct river flux; i.e. the average nutrient flux due to
                  !!      rivers is described by a spatial file, and this is simply applied
                  !!      as a direct nutrient flux (i.e. it does not relate or respond to
                  !!      model runoff)
                  !! nutrient fields are derived from the GlobalNEWS 2 database; carbon and
                  !! alkalinity are derived from continent-scale DIC estimates (Huang et al., 
                  !! 2012) and some Arctic river alkalinity estimates (Katya?)
                  !!      
                  !! as of 19/07/12, riverine nutrients can now be spread vertically across 
                  !! several grid cells rather than just poured into the surface box; this
                  !! block of code is still executed, however, to set up the total amounts
                  !! of nutrient entering via rivers
                  !!
                  !! nitrogen
                  if (jriver_n .eq. 1) then
                     !! river concentration specified; use runoff to calculate input
                     f_riv_n(ji,jj) = f_runoff(ji,jj) * riv_n(ji,jj)
                  elseif (jriver_n .eq. 2) then
                     !! river flux specified; independent of runoff
                     f_riv_n(ji,jj) = riv_n(ji,jj)
                  endif
                  !!
                  !! silicon
                  if (jriver_si .eq. 1) then
                     !! river concentration specified; use runoff to calculate input
                     f_riv_si(ji,jj) = f_runoff(ji,jj) * riv_si(ji,jj)
                  elseif (jriver_si .eq. 2) then
                     !! river flux specified; independent of runoff
                     f_riv_si(ji,jj) = riv_si(ji,jj)
                  endif
                  !!
                  !! carbon
                  if (jriver_c .eq. 1) then
                     !! river concentration specified; use runoff to calculate input
                     f_riv_c(ji,jj) = f_runoff(ji,jj) * riv_c(ji,jj)
                  elseif (jriver_c .eq. 2) then
                     !! river flux specified; independent of runoff
                     f_riv_c(ji,jj) = riv_c(ji,jj)
                  endif
                  !!
                  !! alkalinity
                  if (jriver_alk .eq. 1) then
                     !! river concentration specified; use runoff to calculate input
                     f_riv_alk(ji,jj) = f_runoff(ji,jj) * riv_alk(ji,jj)
                  elseif (jriver_alk .eq. 2) then
                     !! river flux specified; independent of runoff
                     f_riv_alk(ji,jj) = riv_alk(ji,jj)
                  endif

               endif

               !!----------------------------------------------------------------------
               !! Chlorophyll calculations
               !!----------------------------------------------------------------------
               !!
               !! non-diatoms
               fthetan = max(tiny(zchn), (zchn * xxi) / (zphn + tiny(zphn)))
               faln    = xaln * fthetan
               !!
               !! diatoms
               fthetad = max(tiny(zchd), (zchd * xxi) / (zphd + tiny(zphd)))
               fald    = xald * fthetad

               !! report biological calculations
               if (idf.eq.1.AND.idfval.eq.1) then
                  IF (lwp) write (numout,*) '------------------------------'
                  IF (lwp) write (numout,*) 'faln(',jk,') = ', faln
                  IF (lwp) write (numout,*) 'fald(',jk,') = ', fald
               endif

               !!----------------------------------------------------------------------
               !! Phytoplankton light limitation
               !!----------------------------------------------------------------------
               !!
               !! It is assumed xpar is the depth-averaged (vertical layer) PAR 
               !! Light limitation (check self-shading) in W/m2
               !!
               !! Note that there is no temperature dependence in phytoplankton
               !! growth rate or any other function. 
               !! In calculation of Chl/Phy ratio tiny(phyto) is introduced to avoid
               !! NaNs in case of Phy==0.  
               !!
               !! fthetad and fthetan are Chl:C ratio (gChl/gC) in diat and non-diat: 
               !! for 1:1 Chl:P ratio (mgChl/mmolN) theta=0.012
               !!
	       !! AXY (16/07/09)
	       !! temperature for new Eppley style phytoplankton growth
	       loc_T   = tn(ji,jj,jk)
	       fun_T   = 1.066**(1.0 * loc_T)
               if (jphy.eq.1) then
                  xvpnT = xvpn * fun_T
                  xvpdT = xvpd * fun_T
               else
                  xvpnT = xvpn
                  xvpdT = xvpd
               endif
	       !!
               !! non-diatoms
               fchn1   = (xvpnT * xvpnT) + (faln * faln * xpar(ji,jj,jk) * xpar(ji,jj,jk))
               fchn    = xvpnT / sqrt(fchn1)
               fjln    = fchn * faln * xpar(ji,jj,jk) !! non-diatom J term
               !!
               !! diatoms
               fchd1   = (xvpdT * xvpdT) + (fald * fald * xpar(ji,jj,jk) * xpar(ji,jj,jk))
               fchd    = xvpdT / sqrt(fchd1)
               fjld    = fchd * fald * xpar(ji,jj,jk) !! diatom J term
      
               !! report phytoplankton light limitation
               if (idf.eq.1.AND.idfval.eq.1) then
                  IF (lwp) write (numout,*) '------------------------------'
                  IF (lwp) write (numout,*) 'fchn(',jk,') = ', fchn
                  IF (lwp) write (numout,*) 'fchd(',jk,') = ', fchd
                  IF (lwp) write (numout,*) 'fjln(',jk,') = ', fjln
                  IF (lwp) write (numout,*) 'fjld(',jk,') = ', fjld
               endif

               !!----------------------------------------------------------------------
               !! Phytoplankton nutrient limitation
               !!----------------------------------------------------------------------
               !!
               !! non-diatoms (N, Fe)
               fnln = zdin / (zdin + xnln) !! non-diatom Qn term
               ffln = zfer / (zfer + xfln) !! non-diatom Qf term
               !!
               !! diatoms (N, Si, Fe)
               fnld = zdin / (zdin + xnld) !! diatom Qn term
               fsld = zsil / (zsil + xsld) !! diatom Qs term
               ffld = zfer / (zfer + xfld) !! diatom Qf term

               !! report phytoplankton nutrient limitation
               if (idf.eq.1.AND.idfval.eq.1) then
                  IF (lwp) write (numout,*) '------------------------------'
                  IF (lwp) write (numout,*) 'fnln(',jk,') = ', fnln
                  IF (lwp) write (numout,*) 'fnld(',jk,') = ', fnld
                  IF (lwp) write (numout,*) 'ffln(',jk,') = ', ffln
                  IF (lwp) write (numout,*) 'ffld(',jk,') = ', ffld
                  IF (lwp) write (numout,*) 'fsld(',jk,') = ', fsld
               endif

               !!----------------------------------------------------------------------
               !! Primary production (non-diatoms)
               !! (note: still needs multiplying by phytoplankton concentration)
               !!----------------------------------------------------------------------
               !!
	       if (jliebig .eq. 0) then
                  !! multiplicative nutrient limitation
                  fpnlim = fnln * ffln
               elseif (jliebig .eq. 1) then
                  !! Liebig Law (= most limiting) nutrient limitation
                  fpnlim = min(fnln, ffln)
               endif
               fprn = fjln * fpnlim

               !!----------------------------------------------------------------------
               !! Primary production (diatoms)
               !! (note: still needs multiplying by phytoplankton concentration)
               !!
               !! production here is split between nitrogen production and that of
               !! silicon; depending upon the "intracellular" ratio of Si:N, model
               !! diatoms will uptake nitrogen/silicon differentially; this borrows
               !! from the diatom model of Mongin et al. (2006)
               !!----------------------------------------------------------------------
               !!
	       if (jliebig .eq. 0) then
                  !! multiplicative nutrient limitation
                  fpdlim = fnld * ffld
               elseif (jliebig .eq. 1) then
                  !! Liebig Law (= most limiting) nutrient limitation
                  fpdlim = min(fnld, ffld)
               endif
               !!
               !! "intracellular" elemental ratios
               fsin  = zpds / (zphd + tiny(zphd))
               fnsi  = zphd / (zpds + tiny(zpds))
	       !! AXY (23/02/10): these next variables derive from Mongin et al. (2003)
	       fsin1 = 3.0 * xsin0     !! = 0.6
               fnsi1 = 1.0 / fsin1     !! = 1.667
               fnsi2 = 1.0 / xsin0     !! = 5.0
               !!
               !! conditionalities based on ratios
               !! nitrogen (and iron and carbon)
               if (fsin.le.xsin0) then
                  fprd  = 0.0
                  fsld2 = 0.0
               elseif (fsin.lt.fsin1) then
                  fprd  = xuif * ((fsin - xsin0) / fsin) * (fjld * fpdlim)
                  fsld2 = xuif * ((fsin - xsin0) / fsin)
               elseif (fsin.ge.fsin1) then
                  fprd  = (fjld * fpdlim)
                  fsld2 = 1.0
               endif
               !!
               !! silicon
               if (fsin.lt.fnsi1) then
                  fprds = (fjld * fsld)
               elseif (fsin.lt.fnsi2) then
                  fprds = xuif * ((fnsi - xnsi0) / fnsi) * (fjld * fsld)
               else
                  fprds = 0.0
               endif         

               !! report phytoplankton growth (including diatom silicon submodel)
               if (idf.eq.1.AND.idfval.eq.1) then
                  IF (lwp) write (numout,*) '------------------------------'
                  IF (lwp) write (numout,*) 'fsin(',jk,')   = ', fsin
                  IF (lwp) write (numout,*) 'fnsi(',jk,')   = ', fnsi
                  IF (lwp) write (numout,*) 'fsld2(',jk,')  = ', fsld2
                  IF (lwp) write (numout,*) 'fprn(',jk,')   = ', fprn
                  IF (lwp) write (numout,*) 'fprd(',jk,')   = ', fprd
                  IF (lwp) write (numout,*) 'fprds(',jk,')  = ', fprds
               endif

               !!----------------------------------------------------------------------
               !! Mixed layer primary production
               !! this block calculates the amount of primary production that occurs
               !! within the upper mixed layer; this allows the separate diagnosis
               !! of "sub-surface" primary production; it does assume that short-
               !! term variability in mixed layer depth doesn't mess with things
               !! though
               !!----------------------------------------------------------------------
	       !!
               if (fdep1.le.hmld(ji,jj)) then
                  !! this level is entirely in the mixed layer
                  fq0 = 1.0
               elseif (fdep.ge.hmld(ji,jj)) then
                  !! this level is entirely below the mixed layer
                  fq0 = 0.0
               else
                  !! this level straddles the mixed layer
                  fq0 = (hmld(ji,jj) - fdep) / fthk
               endif
               !!
               fprn_ml = (fprn * zphn * fthk * fq0)
               fprd_ml = (fprd * zphd * fthk * fq0)

               !!----------------------------------------------------------------------
               !! More chlorophyll calculations
               !!----------------------------------------------------------------------
               !!
               !! frn = (xthetam / fthetan) * (fprn / (fthetan * xpar(ji,jj,jk)))
               !! frd = (xthetam / fthetad) * (fprd / (fthetad * xpar(ji,jj,jk)))
               frn = (xthetam * fchn * fnln * ffln       ) / (fthetan + tiny(fthetan))
               !! AXY (12/05/09): there's potentially a problem here; fsld, silicic acid 
               !!   limitation, is used in the following line to regulate chlorophyll 
               !!   growth in a manner that is inconsistent with its use in the regulation 
               !!   of biomass growth; the Mongin term term used in growth is more complex
               !!   than the simple multiplicative function used below
               !! frd = (xthetam * fchd * fnld * ffld * fsld) / (fthetad + tiny(fthetad))
               !! AXY (12/05/09): this replacement line uses the new variable, fsld2, to
               !!   regulate chlorophyll growth
               frd = (xthetamd * fchd * fnld * ffld * fsld2) / (fthetad + tiny(fthetad))

               !! report chlorophyll calculations
               if (idf.eq.1.AND.idfval.eq.1) then
                  IF (lwp) write (numout,*) '------------------------------'
                  IF (lwp) write (numout,*) 'fthetan(',jk,') = ', fthetan
                  IF (lwp) write (numout,*) 'fthetad(',jk,') = ', fthetad
                  IF (lwp) write (numout,*) 'frn(',jk,')     = ', frn
                  IF (lwp) write (numout,*) 'frd(',jk,')     = ', frd
               endif

               !!----------------------------------------------------------------------
               !! Zooplankton Grazing 
               !! this code supplements the base grazing model with one that
               !! considers the C:N ratio of grazed food and balances this against
               !! the requirements of zooplankton growth; this model is derived 
               !! from that of Anderson & Pondaven (2003)
               !!
               !! the current version of the code assumes a fixed C:N ratio for
               !! detritus (in contrast to Anderson & Pondaven, 2003), though the
               !! full equations are retained for future extension
               !!----------------------------------------------------------------------
               !!
               !!----------------------------------------------------------------------
               !! Microzooplankton first
               !!----------------------------------------------------------------------
               !!
               fmi1    = (xkmi * xkmi) + (xpmipn * zphn * zphn) + (xpmid * zdet * zdet)
               fmi     = xgmi * zzmi / fmi1
               fgmipn  = fmi * xpmipn * zphn * zphn   !! grazing on non-diatoms
               fgmid   = fmi * xpmid  * zdet * zdet   !! grazing on detrital nitrogen
#if defined key_roam
               fgmidc  = (zdtc / (zdet + tiny(zdet))) * fgmid  !! grazing on detrital carbon
#else
               !! AXY (26/11/08): implicit detrital carbon change
               fgmidc  = xthetad * fgmid              !! grazing on detrital carbon
#endif
               !!
               !! which translates to these incoming N and C fluxes
               finmi   = (1.0 - xphi) * (fgmipn + fgmid)
               ficmi   = (1.0 - xphi) * ((xthetapn * fgmipn) + fgmidc)
               !!
               !! the ideal food C:N ratio for microzooplankton
               !! xbetan = 0.77; xthetaz = 5.625; xbetac = 0.64; xkc = 0.80
               fstarmi = (xbetan * xthetazmi) / (xbetac * xkc)
               !!
               !! process these to determine proportioning of grazed N and C
               !! (since there is no explicit consideration of respiration,
               !! only growth and excretion are calculated here)
               fmith   = (ficmi / (finmi + tiny(finmi)))
               if (fmith.ge.fstarmi) then
                  fmigrow = xbetan * finmi
                  fmiexcr = 0.0
               else
                  fmigrow = (xbetac * xkc * ficmi) / xthetazmi
                  fmiexcr = ficmi * ((xbetan / (fmith + tiny(fmith))) - ((xbetac * xkc) / xthetazmi))
               endif
#if defined key_roam
               fmiresp = (xbetac * ficmi) - (xthetazmi * fmigrow)
#endif

               !! report microzooplankton grazing
               if (idf.eq.1.AND.idfval.eq.1) then
                  IF (lwp) write (numout,*) '------------------------------'
                  IF (lwp) write (numout,*) 'fmi1(',jk,')    = ', fmi1
                  IF (lwp) write (numout,*) 'fmi(',jk,')     = ', fmi
                  IF (lwp) write (numout,*) 'fgmipn(',jk,')  = ', fgmipn
                  IF (lwp) write (numout,*) 'fgmid(',jk,')   = ', fgmid
                  IF (lwp) write (numout,*) 'fgmidc(',jk,')  = ', fgmidc
                  IF (lwp) write (numout,*) 'finmi(',jk,')   = ', finmi
                  IF (lwp) write (numout,*) 'ficmi(',jk,')   = ', ficmi
                  IF (lwp) write (numout,*) 'fstarmi(',jk,') = ', fstarmi
                  IF (lwp) write (numout,*) 'fmith(',jk,')   = ', fmith
                  IF (lwp) write (numout,*) 'fmigrow(',jk,') = ', fmigrow
                  IF (lwp) write (numout,*) 'fmiexcr(',jk,') = ', fmiexcr
#if defined key_roam
                  IF (lwp) write (numout,*) 'fmiresp(',jk,') = ', fmiresp
#endif
               endif

               !!----------------------------------------------------------------------
               !! Mesozooplankton second
               !!----------------------------------------------------------------------
               !!
               fme1    = (xkme * xkme) + (xpmepn * zphn * zphn) + (xpmepd * zphd * zphd) + & 
                         (xpmezmi * zzmi * zzmi) + (xpmed * zdet * zdet)
               fme     = xgme * zzme / fme1
               fgmepn  = fme * xpmepn  * zphn * zphn  !! grazing on non-diatoms
               fgmepd  = fme * xpmepd  * zphd * zphd  !! grazing on diatoms
               fgmepds = fsin * fgmepd                !! grazing on diatom silicon
               fgmezmi = fme * xpmezmi * zzmi * zzmi  !! grazing on microzooplankton
               fgmed   = fme * xpmed   * zdet * zdet  !! grazing on detrital nitrogen
#if defined key_roam
               fgmedc  = (zdtc / (zdet + tiny(zdet))) * fgmed  !! grazing on detrital carbon
#else
               !! AXY (26/11/08): implicit detrital carbon change
               fgmedc  = xthetad * fgmed              !! grazing on detrital carbon
#endif
               !!
               !! which translates to these incoming N and C fluxes
               finme   = (1.0 - xphi) * (fgmepn + fgmepd + fgmezmi + fgmed)
               ficme   = (1.0 - xphi) * ((xthetapn * fgmepn) + (xthetapd * fgmepd) + &
                        (xthetazmi * fgmezmi) + fgmedc)
               !!
               !! the ideal food C:N ratio for mesozooplankton
               !! xbetan = 0.77; xthetaz = 5.625; xbetac = 0.64; xkc = 0.80
               fstarme = (xbetan * xthetazme) / (xbetac * xkc)
               !!
               !! process these to determine proportioning of grazed N and C
               !! (since there is no explicit consideration of respiration,
               !! only growth and excretion are calculated here)
               fmeth   = (ficme / (finme + tiny(finme)))
               if (fmeth.ge.fstarme) then
                  fmegrow = xbetan * finme
                  fmeexcr = 0.0
               else
                  fmegrow = (xbetac * xkc * ficme) / xthetazme
                  fmeexcr = ficme * ((xbetan / (fmeth + tiny(fmeth))) - ((xbetac * xkc) / xthetazme))
               endif
#if defined key_roam
               fmeresp = (xbetac * ficme) - (xthetazme * fmegrow)
#endif

               !! report mesozooplankton grazing
               if (idf.eq.1.AND.idfval.eq.1) then
                  IF (lwp) write (numout,*) '------------------------------'
                  IF (lwp) write (numout,*) 'fme1(',jk,')    = ', fme1
                  IF (lwp) write (numout,*) 'fme(',jk,')     = ', fme
                  IF (lwp) write (numout,*) 'fgmepn(',jk,')  = ', fgmepn
                  IF (lwp) write (numout,*) 'fgmepd(',jk,')  = ', fgmepd
                  IF (lwp) write (numout,*) 'fgmepds(',jk,') = ', fgmepds
                  IF (lwp) write (numout,*) 'fgmezmi(',jk,') = ', fgmezmi
                  IF (lwp) write (numout,*) 'fgmed(',jk,')   = ', fgmed
                  IF (lwp) write (numout,*) 'fgmedc(',jk,')  = ', fgmedc
                  IF (lwp) write (numout,*) 'finme(',jk,')   = ', finme
                  IF (lwp) write (numout,*) 'ficme(',jk,')   = ', ficme
                  IF (lwp) write (numout,*) 'fstarme(',jk,') = ', fstarme
                  IF (lwp) write (numout,*) 'fmeth(',jk,')   = ', fmeth
                  IF (lwp) write (numout,*) 'fmegrow(',jk,') = ', fmegrow
                  IF (lwp) write (numout,*) 'fmeexcr(',jk,') = ', fmeexcr
#if defined key_roam
                  IF (lwp) write (numout,*) 'fmeresp(',jk,') = ', fmeresp
#endif
               endif

               fzmi_i(ji,jj)  = fzmi_i(ji,jj)  + fthk * (  &
                  fgmipn + fgmid )
               fzmi_o(ji,jj)  = fzmi_o(ji,jj)  + fthk * (  &
                  fmigrow + (xphi * (fgmipn + fgmid)) + fmiexcr + ((1.0 - xbetan) * finmi) )
               fzme_i(ji,jj)  = fzme_i(ji,jj)  + fthk * (  &
                  fgmepn + fgmepd + fgmezmi + fgmed )
               fzme_o(ji,jj)  = fzme_o(ji,jj)  + fthk * (  &
                  fmegrow + (xphi * (fgmepn + fgmepd + fgmezmi + fgmed)) + fmeexcr + ((1.0 - xbetan) * finme) )

               !!----------------------------------------------------------------------
               !! Plankton metabolic losses
               !! Linear loss processes assumed to be metabolic in origin
               !!----------------------------------------------------------------------
               !!
	       fdpn2  = xmetapn  * zphn
	       fdpd2  = xmetapd  * zphd
	       fdpds2 = xmetapd  * zpds
	       fdzmi2 = xmetazmi * zzmi
	       fdzme2 = xmetazme * zzme

               !!----------------------------------------------------------------------
               !! Plankton mortality losses
	       !! EKP (26/02/09): phytoplankton hyperbolic mortality term introduced 
               !! to improve performance in gyres
               !!----------------------------------------------------------------------
               !!
               !! non-diatom phytoplankton
	       if (jmpn.eq.1) fdpn = xmpn * zphn               !! linear
	       if (jmpn.eq.2) fdpn = xmpn * zphn * zphn        !! quadratic
	       if (jmpn.eq.3) fdpn = xmpn * zphn * &           !! hyperbolic
                  (zphn / (xkphn + zphn))
	       if (jmpn.eq.4) fdpn = xmpn * zphn * &           !! sigmoid
                  ((zphn * zphn) / (xkphn + (zphn * zphn)))
               !!
               !! diatom phytoplankton
	       if (jmpd.eq.1) fdpd = xmpd * zphd               !! linear
	       if (jmpd.eq.2) fdpd = xmpd * zphd * zphd        !! quadratic
	       if (jmpd.eq.3) fdpd = xmpd * zphd * &           !! hyperbolic
                  (zphd / (xkphd + zphd))
	       if (jmpd.eq.4) fdpd = xmpd * zphd * &           !! sigmoid
                  ((zphd * zphd) / (xkphd + (zphd * zphd)))
               fdpds = fdpd * fsin
               !!
               !! microzooplankton
	       if (jmzmi.eq.1) fdzmi = xmzmi * zzmi            !! linear
	       if (jmzmi.eq.2) fdzmi = xmzmi * zzmi * zzmi     !! quadratic
	       if (jmzmi.eq.3) fdzmi = xmzmi * zzmi * &        !! hyperbolic
                  (zzmi / (xkzmi + zzmi))
	       if (jmzmi.eq.4) fdzmi = xmzmi * zzmi * &        !! sigmoid
                  ((zzmi * zzmi) / (xkzmi + (zzmi * zzmi)))
               !!
               !! mesozooplankton
	       if (jmzme.eq.1) fdzme = xmzme * zzme            !! linear
	       if (jmzme.eq.2) fdzme = xmzme * zzme * zzme     !! quadratic
	       if (jmzme.eq.3) fdzme = xmzme * zzme * &        !! hyperbolic
                  (zzme / (xkzme + zzme))
	       if (jmzme.eq.4) fdzme = xmzme * zzme * &        !! sigmoid
                  ((zzme * zzme) / (xkzme + (zzme * zzme)))

               !!----------------------------------------------------------------------
               !! Detritus remineralisation
               !! Constant or temperature-dependent
               !!----------------------------------------------------------------------
               !!
               if (jmd.eq.1) then
                  !! temperature-dependent
                  fdd  = xmd  * fun_T * zdet
#if defined key_roam
                  fddc = xmdc * fun_T * zdtc
#endif
               else
                  !! temperature-independent
                  fdd  = xmd  * zdet
#if defined key_roam
                  fddc = xmdc * zdtc
#endif
               endif
               !!
               !! AXY (22/07/09): accelerate detrital remineralisation in the bottom box
               if (jk.eq.(mbathy(ji,jj)-1) .and. jsfd.eq.1) then
                  fdd  = 1.0  * zdet
#if defined key_roam
                  fddc = 1.0  * zdtc
#endif
               endif
               
               !! report plankton mortality and remineralisation
               if (idf.eq.1.AND.idfval.eq.1) then
                  IF (lwp) write (numout,*) '------------------------------'
                  IF (lwp) write (numout,*) 'fdpn2(',jk,') = ', fdpn2
                  IF (lwp) write (numout,*) 'fdpd2(',jk,') = ', fdpd2
                  IF (lwp) write (numout,*) 'fdpds2(',jk,')= ', fdpds2
                  IF (lwp) write (numout,*) 'fdzmi2(',jk,')= ', fdzmi2
                  IF (lwp) write (numout,*) 'fdzme2(',jk,')= ', fdzme2
                  IF (lwp) write (numout,*) 'fdpn(',jk,')  = ', fdpn
                  IF (lwp) write (numout,*) 'fdpd(',jk,')  = ', fdpd
                  IF (lwp) write (numout,*) 'fdpds(',jk,') = ', fdpds
                  IF (lwp) write (numout,*) 'fdzmi(',jk,') = ', fdzmi
                  IF (lwp) write (numout,*) 'fdzme(',jk,') = ', fdzme
                  IF (lwp) write (numout,*) 'fdd(',jk,')   = ', fdd
#if defined key_roam
                  IF (lwp) write (numout,*) 'fddc(',jk,')  = ', fddc
#endif
               endif

               !!----------------------------------------------------------------------
               !! Detritus addition to benthos
               !! If activated, slow detritus in the bottom box will enter the
               !! benthic pool
               !!----------------------------------------------------------------------
               !!
               if (jk.eq.(mbathy(ji,jj)-1) .and. jorgben.eq.1) then
                  !! this is the BOTTOM OCEAN BOX -> into the benthic pool!
                  !!
                  f_sbenin_n(ji,jj)  = (zdet * vsed * 86400.)
                  f_sbenin_fe(ji,jj) = (zdet * vsed * 86400. * xrfn)
#if defined key_roam
                  f_sbenin_c(ji,jj)  = (zdtc * vsed * 86400.)
#else
                  f_sbenin_c(ji,jj)  = (zdet * vsed * 86400. * xthetad)
#endif
               endif

               !!----------------------------------------------------------------------
               !! Iron chemistry and fractionation
               !! following the Parekh et al. (2004) scheme adopted by the Met.
               !! Office, Medusa models total iron but considers "free" and
               !! ligand-bound forms for the purposes of scavenging (only "free"
               !! iron can be scavenged
               !!----------------------------------------------------------------------
               !!
               !! total iron concentration (mmol Fe / m3 -> umol Fe / m3)
               xFeT        = zfer * 1.e3
               !!
               !! calculate fractionation (based on Diat-HadOCC; in turn based on Parekh et al., 2004)
               xb_coef_tmp = xk_FeL * (xLgT - xFeT) - 1.0
               xb2M4ac     = max(((xb_coef_tmp * xb_coef_tmp) + (4.0 * xk_FeL * xLgT)), 0.0)
               !!
               !! "free" ligand concentration
               xLgF        = 0.5 * (xb_coef_tmp + (xb2M4ac**0.5)) / xk_FeL
               !!
               !! ligand-bound iron concentration
               xFeL        = xLgT - xLgF
               !!
               !! "free" iron concentration (and convert to mmol Fe / m3)
               xFeF        = (xFeT - xFeL) * 1.e-3
               xFree       = xFeF / (zfer + tiny(zfer))
               !!
               !! scavenging of iron (multiple schemes); I'm only really happy with the 
               !! first one at the moment - the others involve assumptions (sometimes
               !! guessed at by me) that are potentially questionable
	       !!
	       if (jiron.eq.1) then
		  !!----------------------------------------------------------------------
	          !! Scheme 1: Dutkiewicz et al. (2005)
		  !! This scheme includes a single scavenging term based solely on a
                  !! fixed rate and the availablility of "free" iron
		  !!----------------------------------------------------------------------
                  !!
                  ffescav     = xk_sc_Fe * xFeF                     ! = mmol/m3/d
		  !!
		  !!----------------------------------------------------------------------
                  !!
                  !! Mick's code contains a further (optional) implicit "scavenging" of 
                  !! iron that sets an upper bound on "free" iron concentration, and 
                  !! essentially caps the concentration of total iron as xFeL + "free" 
                  !! iron; since the former is constrained by a fixed total ligand 
                  !! concentration (= 1.0 umol/m3), and the latter isn't allowed above 
                  !! this upper bound, total iron is constrained to a maximum of ...
                  !!
                  !!    xFeL + min(xFeF, 0.3 umol/m3) = 1.0 + 0.3 = 1.3 umol / m3
                  !! 
                  !! In Mick's code, the actual value of total iron is reset to this
                  !! sum (i.e. TFe = FeL + Fe'; but Fe' <= 0.3 umol/m3); this isn't
                  !! our favoured approach to tracer updating here (not least because
                  !! of the leapfrog), so here the amount scavenged is augmented by an
                  !! additional amount that serves to drag total iron back towards that
                  !! expected from this limitation on iron concentration ...
                  !!
                  xmaxFeF     = min((xFeF * 1.e3), 0.3)             ! = umol/m3
                  !!
                  !! Here, the difference between current total Fe and (FeL + Fe') is
                  !! calculated and added to the scavenging flux already calculated
                  !! above ...
                  !!
                  fdeltaFe    = (xFeT - (xFeL + xmaxFeF)) * 1.e-3   ! = mmol/m3
                  !!
                  !! This assumes that the "excess" iron is dissipated with a time-
                  !! scale of 1 day; seems reasonable to me ... (famous last words)
                  !!
                  ffescav     = ffescav + fdeltaFe                  ! = mmol/m3/d
                  !!
               elseif (jiron.eq.2) then
		  !!----------------------------------------------------------------------
                  !! Scheme 2: Moore et al. (2004)
                  !! This scheme includes a single scavenging term that accounts for
                  !! both suspended and sinking particles in the water column; this
                  !! term scavenges total iron rather than "free" iron
		  !!----------------------------------------------------------------------
  	          !!
                  !! total iron concentration (mmol Fe / m3 -> umol Fe / m3)
                  xFeT        = zfer * 1.e3
                  !!
                  !! this has a base scavenging rate (12% / y) which is modified by local
                  !! particle concentration and sinking flux (and dust - but I'm ignoring
                  !! that here for now) and which is accelerated when Fe concentration gets
                  !! 0.6 nM (= 0.6 umol/m3 = 0.0006 mmol/m3), and decreased as concentrations
                  !! below 0.4 nM (= 0.4 umol/m3 = 0.0004 mmol/m3)
                  !!
                  !! base scavenging rate (0.12 / y)
                  fbase_scav = 0.12 / 365.25
                  !!
                  !! calculate sinking particle part of scaling factor
                  !! this takes local fast sinking carbon (mmol C / m2 / d) and
                  !! gets it into nmol C / cm3 / s ("rdt" below is the number of seconds in
                  !! a model timestep)
                  !!
                  !! fscal_sink = ffastc(ji,jj) * 1.e2 / (86400.)
                  !!
                  !! ... actually, re-reading Moore et al.'s equations, it looks like he uses
                  !! his sinking flux directly, without scaling it by time-step or anything,
                  !! so I'll copy this here ...
                  !!
                  fscal_sink = ffastc(ji,jj) * 1.e2
                  !!
                  !! calculate particle part of scaling factor
                  !! this totals up the carbon in suspended particles (Pn, Pd, Zmi, Zme, D),
                  !! which comes out in mmol C / m3 (= nmol C / cm3), and then multiplies it
                  !! by a magic factor, 0.002, to get it into nmol C / cm2 / s
                  !!
                  fscal_part = ((xthetapn * zphn) + (xthetapd * zphd) + (xthetazmi * zzmi) + &
                  (xthetazme * zzme) + (xthetad * zdet)) * 0.002
                  !!
                  !! calculate scaling factor for base scavenging rate
                  !! this uses the (now correctly scaled) sinking flux and standing
                  !! particle concentration, divides through by some sort of reference
                  !! value (= 0.0066 nmol C / cm2 / s) and then uses this, or not if its
                  !! too high, to rescale the base scavenging rate
                  !!
                  fscal_scav = fbase_scav * min(((fscal_sink + fscal_part) / 0.0066), 4.0)
                  !!
                  !! the resulting scavenging rate is then scaled further according to the
                  !! local iron concentration (i.e. diminished in low iron regions; enhanced
                  !! in high iron regions; less alone in intermediate iron regions)
                  !!
                  if (xFeT.lt.0.4) then
                     !!
                     !! low iron region
                     !!
                     fscal_scav = fscal_scav * (xFeT / 0.4)
                     !!
                  elseif (xFeT.gt.0.6) then
                     !!
                     !! high iron region
                     !!
                     fscal_scav = fscal_scav + ((xFeT / 0.6) * (6.0 / 1.4))
                     !!
                  else
                     !!
                     !! intermediate iron region: do nothing
                     !!
                  endif
                  !! 
                  !! apply the calculated scavenging rate ...
                  !!
                  ffescav = fscal_scav * zfer
                  !!
               elseif (jiron.eq.3) then
		  !!----------------------------------------------------------------------
                  !! Scheme 3: Moore et al. (2008)
                  !! This scheme includes a single scavenging term that accounts for
                  !! sinking particles in the water column, and includes organic C,
                  !! biogenic opal, calcium carbonate and dust in this (though the
                  !! latter is ignored here until I work out what units the incoming
                  !! "dust" flux is in); this term scavenges total iron rather than 
                  !! "free" iron
		  !!----------------------------------------------------------------------
  	          !!
                  !! total iron concentration (mmol Fe / m3 -> umol Fe / m3)
                  xFeT        = zfer * 1.e3
                  !!
                  !! this has a base scavenging rate which is modified by local
                  !! particle sinking flux (including dust - but I'm ignoring that 
                  !! here for now) and which is accelerated when Fe concentration
                  !! is > 0.6 nM (= 0.6 umol/m3 = 0.0006 mmol/m3), and decreased as 
                  !! concentrations < 0.5 nM (= 0.5 umol/m3 = 0.0005 mmol/m3)
                  !!
                  !! base scavenging rate (Fe_b in paper; units may be wrong there)
                  fbase_scav = 0.00384 ! (ng)^-1 cm
                  !!
                  !! calculate sinking particle part of scaling factor; this converts
                  !! mmol / m2 / d fluxes of organic carbon, silicon and calcium
                  !! carbonate into ng / cm2 / s fluxes; it is assumed here that the
                  !! mass conversions simply consider the mass of the main element
                  !! (C, Si and Ca) and *not* the mass of the molecules that they are
                  !! part of; Moore et al. (2008) is unclear on the conversion that
                  !! should be used
                  !!
                  !! milli -> nano; mol -> gram; /m2 -> /cm2; /d -> /s
                  fscal_csink  = (ffastc(ji,jj)  * 1.e6 * xmassc  * 1.e-4 / 86400.)      ! ng C  / cm2 / s
                  fscal_sisink = (ffastsi(ji,jj) * 1.e6 * xmasssi * 1.e-4 / 86400.)      ! ng Si / cm2 / s
                  fscal_casink = (ffastca(ji,jj) * 1.e6 * xmassca * 1.e-4 / 86400.)      ! ng Ca / cm2 / s
                  !! 
                  !! sum up these sinking fluxes and convert to ng / cm by dividing
                  !! through by a sinking rate of 100 m / d = 1.157 cm / s
                  fscal_sink   = ((fscal_csink * 6.) + fscal_sisink + fscal_casink) / &
                  (100. * 1.e3 / 86400)                                                  ! ng / cm
                  !!
                  !! now calculate the scavenging rate based upon the base rate and
                  !! this particle flux scaling; according to the published units,
                  !! the result actually has *no* units, but as it must be expressed
                  !! per unit time for it to make any sense, I'm assuming a missing
                  !! "per second"
                  fscal_scav = fbase_scav * fscal_sink                                   ! / s
                  !!
                  !! the resulting scavenging rate is then scaled further according to the
                  !! local iron concentration (i.e. diminished in low iron regions; enhanced
                  !! in high iron regions; less alone in intermediate iron regions)
                  !!
                  if (xFeT.lt.0.5) then
                     !!
                     !! low iron region (0.5 instead of the 0.4 in Moore et al., 2004)
                     !!
                     fscal_scav = fscal_scav * (xFeT / 0.5)
                     !!
                  elseif (xFeT.gt.0.6) then
                     !!
                     !! high iron region (functional form different in Moore et al., 2004)
                     !!
                     fscal_scav = fscal_scav + ((xFeT - 0.6) * 0.00904)
                     !!
                  else
                     !!
                     !! intermediate iron region: do nothing
                     !!
                  endif
                  !! 
                  !! apply the calculated scavenging rate ...
                  !!
                  ffescav = fscal_scav * zfer
                  !!
               elseif (jiron.eq.4) then
		  !!----------------------------------------------------------------------
                  !! Scheme 4: Galbraith et al. (2010)
                  !! This scheme includes two scavenging terms, one for organic,
                  !! particle-based scavenging, and another for inorganic scavenging;
                  !! both terms scavenge "free" iron only
		  !!----------------------------------------------------------------------
  	          !!
	          !! Galbraith et al. (2010) present a more straightforward outline of 
                  !! the scheme in Parekh et al. (2005) ...
                  !! 
                  !! sinking particulate carbon available for scavenging
                  !! this assumes a sinking rate of 100 m / d (Moore & Braucher, 2008),
                  xCscav1     = (ffastc(ji,jj) * xmassc) / 100. ! = mg C / m3
                  !! 
                  !! scale by Honeyman et al. (1981) exponent coefficient
                  !! multiply by 1.e-3 to express C flux in g C rather than mg C
                  xCscav2     = (xCscav1 * 1.e-3)**0.58
                  !!
                  !! multiply by Galbraith et al. (2010) scavenging rate
                  xk_org      = 0.5 ! ((g C m/3)^-1) / d
                  xORGscav    = xk_org * xCscav2 * xFeF
                  !!
                  !! Galbraith et al. (2010) also include an inorganic bit ...
                  !! 
                  !! this occurs at a fixed rate, again based on the availability of
                  !! "free" iron
                  !!
                  !! k_inorg  = 1000 d**-1 nmol Fe**-0.5 kg**-0.5
	          !!
	          !! to implement this here, scale xFeF by 1026 to put in units of
                  !! umol/m3 which approximately equal nmol/kg
                  !!
                  xk_inorg    = 1000. ! ((nmol Fe / kg)^1.5)
                  xINORGscav  = (xk_inorg * (xFeF * 1026.)**1.5) * 1.e-3
                  !!
                  !! sum these two terms together
                  ffescav     = xORGscav + xINORGscav
               else
		  !!----------------------------------------------------------------------
                  !! No Scheme: you coward!
                  !! This scheme puts its head in the sand and eskews any decision about
                  !! how iron is removed from the ocean; prepare to get deluged in iron
                  !! you fool!
		  !!----------------------------------------------------------------------
                  ffescav     = 0.
               endif

               !!----------------------------------------------------------------------
               !! Other iron cycle processes
               !!----------------------------------------------------------------------
               !!
               !! aeolian iron deposition
               if (jk.eq.1) then
                  !! dust   is in g Fe / m2 / month
                  !! ffetop is in mmol / m3 / day
                  ffetop  = (((dust(ji,jj) * 1.e3 * xfe_sol) / xfe_mass) / fthk) / 30.
               else
                  ffetop  = 0.0
               endif
               !!
               !! seafloor iron addition
	       !! AXY (10/07/12): amended to only apply sedimentary flux up to ~500 m down
               !! if (jk.eq.(mbathy(ji,jj)-1).AND.jk.lt.i1100) then
               if (jk.eq.(mbathy(ji,jj)-1).AND.jk.le.i0500) then
                  !! Moore et al. (2004) cite a coastal California value of 5 umol/m2/d, but adopt a
                  !! global value of 2 umol/m2/d for all areas < 1100 m; here we use this latter value
                  !! but apply it everywhere
		  !! AXY (21/07/09): actually, let's just apply it below 1100 m (levels 1-37)
                  ffebot  = (xfe_sed / fthk)
               else
                  ffebot  = 0.0
               endif

	       !! AXY (16/12/09): remove iron addition/removal processes
               !! For the purposes of the quarter degree run, the iron cycle is being pegged to the
               !! initial condition supplied by Mick Follows via restoration with a 30 day period;
               !! iron addition at the seafloor is still permitted with the idea that this extra
               !! iron will be removed by the restoration away from the source
               !! ffescav = 0.0
               !! ffetop  = 0.0
               !! ffebot  = 0.0

               !! report miscellaneous calculations
               if (idf.eq.1.AND.idfval.eq.1) then
                  IF (lwp) write (numout,*) '------------------------------'
                  IF (lwp) write (numout,*) 'xfe_sol  = ', xfe_sol
                  IF (lwp) write (numout,*) 'xfe_mass = ', xfe_mass
                  IF (lwp) write (numout,*) 'ffetop(',jk,')  = ', ffetop
                  IF (lwp) write (numout,*) 'ffebot(',jk,')  = ', ffebot
                  IF (lwp) write (numout,*) 'xFree(',jk,')   = ', xFree
                  IF (lwp) write (numout,*) 'ffescav(',jk,') = ', ffescav
               endif

               !!----------------------------------------------------------------------
               !! Miscellaneous
               !!----------------------------------------------------------------------
               !!
               !! diatom frustule dissolution
               fsdiss  = xsdiss * zpds

               !! report miscellaneous calculations
               if (idf.eq.1.AND.idfval.eq.1) then
                  IF (lwp) write (numout,*) '------------------------------'
                  IF (lwp) write (numout,*) 'fsdiss(',jk,')  = ', fsdiss
               endif

               !!----------------------------------------------------------------------
	       !! Slow detritus creation
               !!----------------------------------------------------------------------
	       !! this variable integrates the creation of slow sinking detritus
	       !! to allow the split between fast and slow detritus to be 
	       !! diagnosed
	       fslown  = fdpn + fdzmi + ((1.0 - xfdfrac1) * fdpd) + &
               ((1.0 - xfdfrac2) * fdzme) + ((1.0 - xbetan) * (finmi + finme))
               !!
               !! this variable records the slow detrital sinking flux at this
               !! particular depth; it is used in the output of this flux at
               !! standard depths in the diagnostic outputs; needs to be
               !! adjusted from per second to per day because of parameter vsed
               fslownflux = zdet * vsed * 86400.
#if defined key_roam
               !!
               !! and the same for detrital carbon
	       fslowc  = (xthetapn * fdpn) + (xthetazmi * fdzmi) + &
               (xthetapd * (1.0 - xfdfrac1) * fdpd) + &
               (xthetazme * (1.0 - xfdfrac2) * fdzme) + &
               ((1.0 - xbetac) * (ficmi + ficme))
               !!
               !! this variable records the slow detrital sinking flux at this
               !! particular depth; it is used in the output of this flux at
               !! standard depths in the diagnostic outputs; needs to be
               !! adjusted from per second to per day because of parameter vsed
               fslowcflux = zdtc * vsed * 86400.
#endif

               !!----------------------------------------------------------------------
               !! Nutrient regeneration
               !! this variable integrates total nitrogen regeneration down the
               !! watercolumn; its value is stored and output as a 2D diagnostic;
               !! the corresponding dissolution flux of silicon (from sources
               !! other than fast detritus) is also integrated; note that,
               !! confusingly, the linear loss terms from plankton compartments
               !! are labelled as fdX2 when one might have expected fdX or fdX1
               !!----------------------------------------------------------------------
               !!
               !! nitrogen
	       fregen   = (( (xphi * (fgmipn + fgmid)) +                &  ! messy feeding
               (xphi * (fgmepn + fgmepd + fgmezmi + fgmed)) +           &  ! messy feeding
               fmiexcr + fmeexcr + fdd +                                &  ! excretion + D remin.
	       fdpn2 + fdpd2 + fdzmi2 + fdzme2) * fthk)                    ! linear mortality
               !!
               !! silicon
               fregensi = (( fsdiss + ((1.0 - xfdfrac1) * fdpds) +      &  ! dissolution + non-lin. mortality
               ((1.0 - xfdfrac3) * fgmepds) +                           &  ! egestion by zooplankton
	       fdpds2) * fthk)                                             ! linear mortality
#if defined key_roam
               !!
               !! carbon
               fregenc  = (( (xphi * ((xthetapn * fgmipn) + fgmidc)) +  &  ! messy feeding
               (xphi * ((xthetapn * fgmepn) + (xthetapd * fgmepd) +     &  ! messy feeding
               (xthetazmi * fgmezmi) + fgmedc)) +                       &  ! messy feeding
               fmiresp + fmeresp + fddc +                               &  ! respiration + D remin.
               (xthetapn * fdpn2) + (xthetapd * fdpd2) +                &  ! linear mortality
               (xthetazmi * fdzmi2) + (xthetazme * fdzme2)) * fthk)        ! linear mortality
#endif

               !!----------------------------------------------------------------------
               !! Fast-sinking detritus terms
               !! "local" variables declared so that conservation can be checked;
               !! the calculated terms are added to the fast-sinking flux later on
               !! only after the flux entering this level has experienced some
               !! remineralisation
               !! note: these fluxes need to be scaled by the level thickness
               !!----------------------------------------------------------------------
               !!
               !! nitrogen:   diatom and mesozooplankton mortality
               ftempn         = b0 * ((xfdfrac1 * fdpd)  + (xfdfrac2 * fdzme))
               !!
               !! silicon:    diatom mortality and grazed diatoms
               ftempsi        = b0 * ((xfdfrac1 * fdpds) + (xfdfrac3 * fgmepds))
               !!
               !! iron:       diatom and mesozooplankton mortality
               ftempfe        = b0 * (((xfdfrac1 * fdpd) + (xfdfrac2 * fdzme)) * xrfn)
               !!
               !! carbon:     diatom and mesozooplankton mortality
               ftempc         = b0 * ((xfdfrac1 * xthetapd * fdpd) + & 
                                (xfdfrac2 * xthetazme * fdzme))
               !!
#if defined key_roam
               if (jrratio.eq.0) then
                  !! CaCO3:      latitudinally-based fraction of total primary production
                  !!               absolute latitude of current grid cell
                  flat           = abs(gphit(ji,jj))
                  !!               0.10 at equator; 0.02 at pole
                  fcaco3         = xcaco3a + ((xcaco3b - xcaco3a) * ((90.0 - flat) / 90.0))
               elseif (jrratio.eq.1) then
                  !! CaCO3:      Ridgwell et al. (2007) submodel, version 1
                  !!             this uses SURFACE omega calcite to regulate rain ratio
                  if (f_omcal(ji,jj).ge.1.0) then
                     fq1 = (f_omcal(ji,jj) - 1.0)**0.81
                  else
                     fq1 = 0.
                  endif
                  fcaco3 = xridg_r0 * fq1
               elseif (jrratio.eq.2) then
                  !! CaCO3:      Ridgwell et al. (2007) submodel, version 2
                  !!             this uses FULL 3D omega calcite to regulate rain ratio
                  if (f3_omcal(ji,jj,jk).ge.1.0) then
                     fq1 = (f3_omcal(ji,jj,jk) - 1.0)**0.81
                  else
                     fq1 = 0.
                  endif
                  fcaco3 = xridg_r0 * fq1
               endif
#else
               !! CaCO3:      latitudinally-based fraction of total primary production
               !!               absolute latitude of current grid cell
               flat           = abs(gphit(ji,jj))
               !!               0.10 at equator; 0.02 at pole
               fcaco3         = xcaco3a + ((xcaco3b - xcaco3a) * ((90.0 - flat) / 90.0))
#endif
	       !! AXY (09/03/09): convert CaCO3 production from function of 
               !! primary production into a function of fast-sinking material; 
               !! technically, this is what Dunne et al. (2007) do anyway; they 
               !! convert total primary production estimated from surface 
               !! chlorophyll to an export flux for which they apply conversion 
               !! factors to estimate the various elemental fractions (Si, Ca)
               ftempca        = ftempc * fcaco3

               !! integrate total fast detritus production
               if (idf.eq.1) then
                  fifd_n(ji,jj)  = fifd_n(ji,jj)  + (ftempn  * fthk)
                  fifd_si(ji,jj) = fifd_si(ji,jj) + (ftempsi * fthk)
                  fifd_fe(ji,jj) = fifd_fe(ji,jj) + (ftempfe * fthk)
#if defined key_roam
                  fifd_c(ji,jj)  = fifd_c(ji,jj)  + (ftempc  * fthk)
#endif
               endif

               !! report quantities of fast-sinking detritus for each component
               if (idf.eq.1.AND.idfval.eq.1) then
                  IF (lwp) write (numout,*) '------------------------------'
                  IF (lwp) write (numout,*) 'fdpd(',jk,')    = ', fdpd
                  IF (lwp) write (numout,*) 'fdzme(',jk,')   = ', fdzme
                  IF (lwp) write (numout,*) 'ftempn(',jk,')  = ', ftempn
                  IF (lwp) write (numout,*) 'ftempsi(',jk,') = ', ftempsi
                  IF (lwp) write (numout,*) 'ftempfe(',jk,') = ', ftempfe
                  IF (lwp) write (numout,*) 'ftempc(',jk,')  = ', ftempc
                  IF (lwp) write (numout,*) 'ftempca(',jk,') = ', ftempca
		  IF (lwp) write (numout,*) 'flat(',jk,')    = ', flat
		  IF (lwp) write (numout,*) 'fcaco3(',jk,')  = ', fcaco3
               endif

               !!----------------------------------------------------------------------
	       !! This version of MEDUSA offers a choice of three methods for
	       !! handling the remineralisation of fast detritus.  All three
               !! do so in broadly the same way:
               !!
               !!   1.  Fast detritus is stored as a 2D array                   [ ffastX  ]
               !!   2.  Fast detritus is added level-by-level                   [ ftempX  ]
               !!   3.  Fast detritus is not remineralised in the top box       [ freminX ]
               !!   4.  Remaining fast detritus is remineralised in the bottom  [ fsedX   ]
               !!       box
               !!
               !! The three remineralisation methods are:
               !!   
               !!   1.  Ballast model (i.e. that published in Yool et al., 2011)
               !!  (1b. Ballast-sans-ballast model)
               !!   2.  Martin et al. (1987)
               !!   3.  Henson et al. (2011)
               !! 
               !! The first of these couples C, N and Fe remineralisation to
               !! the remineralisation of particulate Si and CaCO3, but the 
               !! latter two treat remineralisation of C, N, Fe, Si and CaCO3
               !! completely separately.  At present a switch within the code
               !! regulates which submodel is used, but this should be moved
               !! to the namelist file.
               !! 
               !! The ballast-sans-ballast submodel is an original development
               !! feature of MEDUSA in which the ballast submodel's general
               !! framework and parameterisation is used, but in which there
               !! is no protection of organic material afforded by ballasting
               !! minerals.  While similar, it is not the same as the Martin 
               !! et al. (1987) submodel.
               !!
               !! Since the three submodels behave the same in terms of
               !! accumulating sinking material and remineralising it all at
               !! the seafloor, these portions of the code below are common to
               !! all three.
               !!----------------------------------------------------------------------

               if (jexport.eq.1) then
                  !!======================================================================
                  !! BALLAST SUBMODEL
                  !!======================================================================
                  !! 
                  !!----------------------------------------------------------------------
                  !! Fast-sinking detritus fluxes, pt. 1: REMINERALISATION
                  !! aside from explicitly modelled, slow-sinking detritus, the
                  !! model includes an implicit representation of detrital
                  !! particles that sink too quickly to be modelled with
                  !! explicit state variables; this sinking flux is instead
                  !! instantaneously remineralised down the water column using
                  !! the version of Armstrong et al. (2002)'s ballast model
                  !! used by Dunne et al. (2007); the version of this model
                  !! here considers silicon and calcium carbonate ballast
                  !! minerals; this section of the code redistributes the fast
                  !! sinking material generated locally down the water column;
                  !! this differs from Dunne et al. (2007) in that fast sinking
                  !! material is distributed at *every* level below that it is
                  !! generated, rather than at every level below some fixed
                  !! depth; this scheme is also different in that sinking material 
                  !! generated in one level is aggregated with that generated by
                  !! shallower levels; this should make the ballast model more
                  !! self-consistent (famous last words)
                  !!----------------------------------------------------------------------
                  !!
                  if (jk.eq.1) then
                     !! this is the SURFACE OCEAN BOX (no remineralisation)
                     !!
                     freminc  = 0.0
                     freminn  = 0.0
                     freminfe = 0.0
                     freminsi = 0.0
                     freminca = 0.0
                  elseif (jk.lt.(mbathy(ji,jj))) then
                     !! this is an OCEAN BOX (remineralise some material)
                     !!
                     !! set up CCD depth to be used depending on user choice
                     if (jocalccd.eq.0) then
                        !! use default CCD field
                        fccd_dep = ocal_ccd(ji,jj)
                     elseif (jocalccd.eq.1) then
                        !! use calculated CCD field
                        fccd_dep = f2_ccd_cal(ji,jj)
                     endif
                     !!
                     !! === organic carbon ===
                     fq0      = ffastc(ji,jj)                        !! how much organic C enters this box        (mol)
                     if (iball.eq.1) then
                        fq1      = (fq0 * xmassc)                    !! how much it weighs                        (mass)
                        fq2      = (ffastca(ji,jj) * xmassca)        !! how much CaCO3 enters this box            (mass)
                        fq3      = (ffastsi(ji,jj) * xmasssi)        !! how much  opal enters this box            (mass)
                        fq4      = (fq2 * xprotca) + (fq3 * xprotsi) !! total protected organic C                 (mass)
                        !! this next term is calculated for C but used for N and Fe as well
                        !! it needs to be protected in case ALL C is protected
                        if (fq4.lt.fq1) then
                           fprotf   = (fq4 / fq1)                    !! protected fraction of total organic C     (non-dim)
                        else
                           fprotf   = 1.0                            !! all organic C is protected                (non-dim)
                        endif
                        fq5      = (1.0 - fprotf)                    !! unprotected fraction of total organic C   (non-dim)
                        fq6      = (fq0 * fq5)                       !! how much organic C is unprotected         (mol)
                        fq7      = (fq6 * exp(-(fthk / xfastc)))     !! how much unprotected C leaves this box    (mol)
                        fq8      = (fq7 + (fq0 * fprotf))            !! how much total C leaves this box          (mol)
                        freminc  = (fq0 - fq8) / fthk                !! C remineralisation in this box            (mol)
                        ffastc(ji,jj) = fq8                          
                        !! report in/out/remin fluxes of carbon for this level
                           if (idf.eq.1.AND.idfval.eq.1) then
                              IF (lwp) write (numout,*) '------------------------------'
                              IF (lwp) write (numout,*) 'totalC(',jk,')  = ', fq1
                              IF (lwp) write (numout,*) 'prtctC(',jk,')  = ', fq4
                              IF (lwp) write (numout,*) 'fprotf(',jk,')  = ', fprotf
                              IF (lwp) write (numout,*) '------------------------------'
                              IF (lwp) write (numout,*) 'IN   C(',jk,')  = ', fq0
                              IF (lwp) write (numout,*) 'LOST C(',jk,')  = ', freminc * fthk
                              IF (lwp) write (numout,*) 'OUT  C(',jk,')  = ', fq8
                              IF (lwp) write (numout,*) 'NEW  C(',jk,')  = ', ftempc * fthk
                           endif
                        else
                        fq1      = fq0 * exp(-(fthk / xfastc))       !! how much organic C leaves this box        (mol)
                        freminc  = (fq0 - fq1) / fthk                !! C remineralisation in this box            (mol)
                        ffastc(ji,jj)  = fq1
                     endif
                     !!
                     !! === organic nitrogen ===
                     fq0      = ffastn(ji,jj)                        !! how much organic N enters this box        (mol)
                     if (iball.eq.1) then
                        fq5      = (1.0 - fprotf)                    !! unprotected fraction of total organic N   (non-dim)
                        fq6      = (fq0 * fq5)                       !! how much organic N is unprotected         (mol)
                        fq7      = (fq6 * exp(-(fthk / xfastc)))     !! how much unprotected N leaves this box    (mol)
                        fq8      = (fq7 + (fq0 * fprotf))            !! how much total N leaves this box          (mol)
                        freminn  = (fq0 - fq8) / fthk                !! N remineralisation in this box            (mol)
                        ffastn(ji,jj)  = fq8
                        !! report in/out/remin fluxes of carbon for this level
                        if (idf.eq.1.AND.idfval.eq.1) then
                           IF (lwp) write (numout,*) '------------------------------'
                           IF (lwp) write (numout,*) 'totalN(',jk,')  = ', fq1
                           IF (lwp) write (numout,*) 'prtctN(',jk,')  = ', fq4
                           IF (lwp) write (numout,*) 'fprotf(',jk,')  = ', fprotf
                           IF (lwp) write (numout,*) '------------------------------'
                           if (freminn < 0.0) then
                              IF (lwp) write (numout,*) '** FREMIN ERROR **'
                           endif
                           IF (lwp) write (numout,*) 'IN   N(',jk,')  = ', fq0
                           IF (lwp) write (numout,*) 'LOST N(',jk,')  = ', freminn * fthk
                           IF (lwp) write (numout,*) 'OUT  N(',jk,')  = ', fq8
                           IF (lwp) write (numout,*) 'NEW  N(',jk,')  = ', ftempn * fthk
                        endif
                     else
                        fq1      = fq0 * exp(-(fthk / xfastc))       !! how much organic N leaves this box        (mol)
                        freminn  = (fq0 - fq1) / fthk                !! N remineralisation in this box            (mol)
                        ffastn(ji,jj)  = fq1
                     endif
                     !!
                     !! === organic iron ===
                     fq0      = ffastfe(ji,jj)                       !! how much organic Fe enters this box       (mol)
                     if (iball.eq.1) then
                        fq5      = (1.0 - fprotf)                    !! unprotected fraction of total organic Fe  (non-dim)
                        fq6      = (fq0 * fq5)                       !! how much organic Fe is unprotected        (mol)
                        fq7      = (fq6 * exp(-(fthk / xfastc)))     !! how much unprotected Fe leaves this box   (mol)
                        fq8      = (fq7 + (fq0 * fprotf))            !! how much total Fe leaves this box         (mol)            
                        freminfe = (fq0 - fq8) / fthk                !! Fe remineralisation in this box           (mol)
                        ffastfe(ji,jj) = fq8
                     else
                        fq1      = fq0 * exp(-(fthk / xfastc))       !! how much total Fe leaves this box         (mol)      
                        freminfe = (fq0 - fq1) / fthk                !! Fe remineralisation in this box           (mol)
                        ffastfe(ji,jj) = fq1
                     endif
                     !!
                     !! === biogenic silicon ===
                     fq0      = ffastsi(ji,jj)                       !! how much  opal centers this box           (mol) 
                     fq1      = fq0 * exp(-(fthk / xfastsi))         !! how much  opal leaves this box            (mol)
                     freminsi = (fq0 - fq1) / fthk                   !! Si remineralisation in this box           (mol)
                     ffastsi(ji,jj) = fq1
                     !!
                     !! === biogenic calcium carbonate ===
                     fq0      = ffastca(ji,jj)                       !! how much CaCO3 enters this box            (mol)
                     if (fdep.le.fccd_dep) then
                        !! whole grid cell above CCD
                        fq1      = fq0                               !! above lysocline, no Ca dissolves          (mol)
                        freminca = 0.0                               !! above lysocline, no Ca dissolves          (mol)
                        fccd(ji,jj) = real(jk)                       !! which is the last level above the CCD?    (#)
                     elseif (fdep.ge.fccd_dep) then
                        !! whole grid cell below CCD
                        fq1      = fq0 * exp(-(fthk / xfastca))      !! how much CaCO3 leaves this box            (mol)
                        freminca = (fq0 - fq1) / fthk                !! Ca remineralisation in this box           (mol)
                     else
                        !! partial grid cell below CCD
                        fq2      = fdep1 - ocal_ccd(ji,jj)           !! amount of grid cell below CCD             (m)
                        fq1      = fq0 * exp(-(fq2 / xfastca))       !! how much CaCO3 leaves this box            (mol)
                        freminca = (fq0 - fq1) / fthk                !! Ca remineralisation in this box           (mol)
                     endif
                     ffastca(ji,jj) = fq1 
                  else
                     !! this is BELOW THE LAST OCEAN BOX (do nothing)
                     freminc  = 0.0
                     freminn  = 0.0
                     freminfe = 0.0
                     freminsi = 0.0
                     freminca = 0.0              
                  endif

               elseif (jexport.eq.2.or.jexport.eq.3) then
                  if (jexport.eq.2) then
                     !!======================================================================
                     !! MARTIN ET AL. (1987) SUBMODEL
                     !!======================================================================
                     !! 
                     !!----------------------------------------------------------------------
                     !! This submodel uses the classic Martin et al. (1987) curve
                     !! to determine the attenuation of fast-sinking detritus down
                     !! the water column.  All three organic elements, C, N and Fe,
                     !! are handled identically, and their quantities in sinking
                     !! particles attenuate according to a power relationship
                     !! governed by parameter "b".  This is assigned a canonical 
                     !! value of -0.858.  Biogenic opal and calcium carbonate are
                     !! attentuated using the same function as in the ballast
                     !! submodel
                     !!----------------------------------------------------------------------
                     !!
                     fb_val = -0.858
                  elseif (jexport.eq.3) then
                     !!======================================================================
                     !! HENSON ET AL. (2011) SUBMODEL
                     !!======================================================================
                     !!
                     !!----------------------------------------------------------------------
                     !! This submodel reconfigures the Martin et al. (1987) curve by
                     !! allowing the "b" value to vary geographically.  Its value is
                     !! set, following Henson et al. (2011), as a function of local
                     !! sea surface temperature:
                     !!   b = -1.06 + (0.024 * SST)
                     !! This means that remineralisation length scales are longer in
                     !! warm, tropical areas and shorter in cold, polar areas.  This
                     !! does seem back-to-front (i.e. one would expect GREATER
                     !! remineralisation in warmer waters), but is an outcome of 
                     !! analysis of sediment trap data, and it may reflect details
                     !! of ecosystem structure that pertain to particle production
                     !! rather than simply Q10.
                     !!----------------------------------------------------------------------
                     !!
                     fl_sst = tn(ji,jj,1)
                     fb_val = -1.06 + (0.024 * fl_sst)
                  endif
                  !!   
                  if (jk.eq.1) then
                     !! this is the SURFACE OCEAN BOX (no remineralisation)
                     !!
                     freminc  = 0.0
                     freminn  = 0.0
                     freminfe = 0.0
                     freminsi = 0.0
                     freminca = 0.0
                  elseif (jk.lt.(mbathy(ji,jj))) then
                     !! this is an OCEAN BOX (remineralise some material)
                     !!
                     !! === organic carbon ===
                     fq0      = ffastc(ji,jj)                        !! how much organic C enters this box        (mol)
                     fq1      = fq0 * ((fdep1/fdep)**fb_val)         !! how much organic C leaves this box        (mol)
                     freminc  = (fq0 - fq1) / fthk                   !! C remineralisation in this box            (mol)
                     ffastc(ji,jj)  = fq1
                     !!
                     !! === organic nitrogen ===
                     fq0      = ffastn(ji,jj)                        !! how much organic N enters this box        (mol)
                     fq1      = fq0 * ((fdep1/fdep)**fb_val)         !! how much organic N leaves this box        (mol)
                     freminn  = (fq0 - fq1) / fthk                   !! N remineralisation in this box            (mol)
                     ffastn(ji,jj)  = fq1
                     !!
                     !! === organic iron ===
                     fq0      = ffastfe(ji,jj)                       !! how much organic Fe enters this box       (mol)
                     fq1      = fq0 * ((fdep1/fdep)**fb_val)         !! how much organic Fe leaves this box       (mol)
                     freminfe = (fq0 - fq1) / fthk                   !! Fe remineralisation in this box           (mol)
                     ffastfe(ji,jj) = fq1
                     !!
                     !! === biogenic silicon ===
                     fq0      = ffastsi(ji,jj)                       !! how much  opal centers this box           (mol) 
                     fq1      = fq0 * exp(-(fthk / xfastsi))         !! how much  opal leaves this box            (mol)
                     freminsi = (fq0 - fq1) / fthk                   !! Si remineralisation in this box           (mol)
                     ffastsi(ji,jj) = fq1
                     !!
                     !! === biogenic calcium carbonate ===
                     fq0      = ffastca(ji,jj)                       !! how much CaCO3 enters this box            (mol)
                     if (fdep.le.ocal_ccd(ji,jj)) then
                        !! whole grid cell above CCD
                        fq1      = fq0                               !! above lysocline, no Ca dissolves          (mol)
                        freminca = 0.0                               !! above lysocline, no Ca dissolves          (mol)
                        fccd(ji,jj) = real(jk)                       !! which is the last level above the CCD?    (#)
                     elseif (fdep.ge.ocal_ccd(ji,jj)) then
                        !! whole grid cell below CCD
                        fq1      = fq0 * exp(-(fthk / xfastca))      !! how much CaCO3 leaves this box            (mol)
                        freminca = (fq0 - fq1) / fthk                !! Ca remineralisation in this box           (mol)
                     else
                        !! partial grid cell below CCD
                        fq2      = fdep1 - ocal_ccd(ji,jj)           !! amount of grid cell below CCD             (m)
                        fq1      = fq0 * exp(-(fq2 / xfastca))       !! how much CaCO3 leaves this box            (mol)
                        freminca = (fq0 - fq1) / fthk                !! Ca remineralisation in this box           (mol)
                     endif
                     ffastca(ji,jj) = fq1 
                  else
                     !! this is BELOW THE LAST OCEAN BOX (do nothing)
                     freminc  = 0.0
                     freminn  = 0.0
                     freminfe = 0.0
                     freminsi = 0.0
                     freminca = 0.0              
                  endif

               endif

               !!----------------------------------------------------------------------
               !! Fast-sinking detritus fluxes, pt. 2: UPDATE FAST FLUXES
               !! here locally calculated additions to the fast-sinking flux are added
               !! to the total fast-sinking flux; this is done here such that material
               !! produced in a particular layer is only remineralised below this
               !! layer
               !!----------------------------------------------------------------------
               !!
               !! add sinking material generated in this layer to running totals
               !!
               !! === organic carbon ===                          (diatom and mesozooplankton mortality)
               ffastc(ji,jj)  = ffastc(ji,jj)  + (ftempc  * fthk)
               !!
               !! === organic nitrogen ===                        (diatom and mesozooplankton mortality)
               ffastn(ji,jj)  = ffastn(ji,jj)  + (ftempn  * fthk)
               !!
               !! === organic iron ===                            (diatom and mesozooplankton mortality)
               ffastfe(ji,jj) = ffastfe(ji,jj) + (ftempfe * fthk)
               !!
               !! === biogenic silicon ===                        (diatom mortality and grazed diatoms)
               ffastsi(ji,jj) = ffastsi(ji,jj) + (ftempsi * fthk)
               !!
               !! === biogenic calcium carbonate ===              (latitudinally-based fraction of total primary production)
               ffastca(ji,jj) = ffastca(ji,jj) + (ftempca * fthk)

               !!----------------------------------------------------------------------
               !! Fast-sinking detritus fluxes, pt. 3: SEAFLOOR
               !! remineralise all remaining fast-sinking detritus to dissolved
               !! nutrients; the sedimentation fluxes calculated here allow the
               !! separation of what's remineralised sinking through the final
               !! ocean box from that which is added to the final box by the
               !! remineralisation of material that reaches the seafloor (i.e.
               !! the model assumes that *all* material that hits the seafloor
               !! is remineralised and that none is permanently buried; hey,
               !! this is a giant GCM model that can't be run for long enough
               !! to deal with burial fluxes!)
	       !!
	       !! in a change to this process, in part so that MEDUSA behaves
	       !! a little more like ERSEM et al., fast-sinking detritus (N, Fe
               !! and C) is converted to slow sinking detritus at the seafloor
               !! instead of being remineralised; the rationale is that in
               !! shallower shelf regions (... that are not fully mixed!) this
               !! allows the detrital material to return slowly to dissolved 
               !! nutrient rather than instantaneously as now; the alternative
               !! would be to explicitly handle seafloor organic material - a
               !! headache I don't wish to experience at this point; note that
               !! fast-sinking Si and Ca detritus is just remineralised as 
               !! per usual
	       !! 
	       !! AXY (13/01/12)
	       !! in a further change to this process, again so that MEDUSA is
               !! a little more like ERSEM et al., material that reaches the
               !! seafloor can now be added to sediment pools and stored for
               !! slow release; there are new 2D arrays for organic nitrogen,
               !! iron and carbon and inorganic silicon and carbon that allow
               !! fast and slow detritus that reaches the seafloor to be held
               !! and released back to the water column more slowly; these arrays
               !! are transferred via the tracer restart files between repeat
               !! submissions of the model
               !!----------------------------------------------------------------------
               !! 
               ffast2slowc  = 0.0
               ffast2slown  = 0.0
               ffast2slowfe = 0.0
               !!
               if (jk.eq.(mbathy(ji,jj)-1)) then
                  !! this is the BOTTOM OCEAN BOX (remineralise everything)
                  !!
                  !! AXY (17/01/12): tweaked to include benthos pools
                  !! 
                  !! === organic carbon ===
                  if (jfdfate.eq.0 .and. jorgben.eq.0) then
                     freminc  = freminc + (ffastc(ji,jj) / fthk)    !! C remineralisation in this box            (mol/m3)
                  elseif (jfdfate.eq.1 .and. jorgben.eq.0) then
                     ffast2slowc = ffastc(ji,jj) / fthk             !! fast C -> slow C                          (mol/m3)
                     fslowc      = fslowc + ffast2slowc
                  elseif (jfdfate.eq.0 .and. jorgben.eq.1) then
                     f_fbenin_c(ji,jj)  = ffastc(ji,jj)             !! fast C -> benthic C                       (mol/m2)
                  endif
                  fsedc    = ffastc(ji,jj)                          !! record seafloor C                         (mol/m2)
                  ffastc(ji,jj)  = 0.0
                  !!
                  !! === organic nitrogen ===
                  if (jfdfate.eq.0 .and. jorgben.eq.0) then
                     freminn  = freminn + (ffastn(ji,jj) / fthk)    !! N remineralisation in this box            (mol/m3)
                  elseif (jfdfate.eq.1 .and. jorgben.eq.0) then
                     ffast2slown = ffastn(ji,jj) / fthk             !! fast N -> slow N                          (mol/m3)
                     fslown      = fslown + ffast2slown
                  elseif (jfdfate.eq.0 .and. jorgben.eq.1) then
                     f_fbenin_n(ji,jj)  = ffastn(ji,jj)             !! fast N -> benthic N                       (mol/m2)
                  endif
                  fsedn    = ffastn(ji,jj)                          !! record seafloor N                         (mol/m2)
                  ffastn(ji,jj)  = 0.0
                  !!
                  !! === organic iron ===
                  if (jfdfate.eq.0 .and. jorgben.eq.0) then
                     freminfe = freminfe + (ffastfe(ji,jj) / fthk)  !! Fe remineralisation in this box           (mol/m3)
                  elseif (jfdfate.eq.1 .and. jorgben.eq.0) then
                     ffast2slowfe = ffastn(ji,jj) / fthk            !! fast Fe -> slow Fe                        (mol/m3)
                  elseif (jfdfate.eq.0 .and. jorgben.eq.1) then
                     f_fbenin_fe(ji,jj) = ffastfe(ji,jj)            !! fast Fe -> benthic Fe                     (mol/m2)
                  endif
                  fsedfe   = ffastfe(ji,jj)                         !! record seafloor Fe                        (mol/m2)
                  ffastfe(ji,jj) = 0.0
                  !!
                  !! === biogenic silicon ===
                  if (jinorgben.eq.0) then
                     freminsi = freminsi + (ffastsi(ji,jj) / fthk)  !! Si remineralisation in this box           (mol/m3)
                  elseif (jinorgben.eq.1) then
                     f_fbenin_si(ji,jj) = ffastsi(ji,jj)            !! fast Si -> benthic Si                     (mol/m2)
                  endif
                  fsedsi   = ffastsi(ji,jj)                         !! record seafloor Si                        (mol/m2)
                  ffastsi(ji,jj) = 0.0
                  !!
                  !! === biogenic calcium carbonate ===
                  if (jinorgben.eq.0) then
                     freminca = freminca + (ffastca(ji,jj) / fthk)  !! Ca remineralisation in this box           (mol/m3) 
                  elseif (jinorgben.eq.1) then
                     f_fbenin_ca(ji,jj) = ffastca(ji,jj)            !! fast Ca -> benthic Ca                     (mol/m2)
                  endif
                  fsedca   = ffastca(ji,jj)                         !! record seafloor Ca                        (mol/m2)
                  ffastca(ji,jj) = 0.0
               endif

               if (idf.eq.1) then
                  !!----------------------------------------------------------------------
                  !! Integrate total fast detritus remineralisation
                  !!----------------------------------------------------------------------
                  !!
                  fofd_n(ji,jj)  = fofd_n(ji,jj)  + (freminn  * fthk)
                  fofd_si(ji,jj) = fofd_si(ji,jj) + (freminsi * fthk)
                  fofd_fe(ji,jj) = fofd_fe(ji,jj) + (freminfe * fthk)
#if defined key_roam
                  fofd_c(ji,jj)  = fofd_c(ji,jj)  + (freminc  * fthk)
#endif
               endif

               !!----------------------------------------------------------------------
               !! Sort out remineralisation tally of fast-sinking detritus
               !!----------------------------------------------------------------------
               !!
	       !! update fast-sinking regeneration arrays
	       fregenfast(ji,jj)   = fregenfast(ji,jj)   + (freminn  * fthk)
	       fregenfastsi(ji,jj) = fregenfastsi(ji,jj) + (freminsi * fthk)
#if defined key_roam
               fregenfastc(ji,jj)  = fregenfastc(ji,jj)  + (freminc  * fthk)
#endif

               !!----------------------------------------------------------------------
               !! Benthic remineralisation fluxes
               !!----------------------------------------------------------------------
               !!
               if (jk.eq.(mbathy(ji,jj)-1)) then
                  !!
                  !! organic components
                  if (jorgben.eq.1) then
                     f_benout_n(ji,jj)  = xsedn  * zn_sed_n(ji,jj)
                     f_benout_fe(ji,jj) = xsedfe * zn_sed_fe(ji,jj)
                     f_benout_c(ji,jj)  = xsedc  * zn_sed_c(ji,jj)
                  endif
                  !!
                  !! inorganic components
                  if (jinorgben.eq.1) then
                     f_benout_si(ji,jj) = xsedsi * zn_sed_si(ji,jj)
                     f_benout_ca(ji,jj) = xsedca * zn_sed_ca(ji,jj)
                     !!
                     !! account for CaCO3 that dissolves when it shouldn't
                     if ( fdep .le. fccd_dep ) then
                        f_benout_lyso_ca(ji,jj) = xsedca * zn_sed_ca(ji,jj)
                     endif
                  endif
               endif

               !!======================================================================
               !! LOCAL GRID CELL TRENDS
               !!======================================================================
               !!
               !!----------------------------------------------------------------------
               !! Determination of trends
               !!----------------------------------------------------------------------
               !!
               !!----------------------------------------------------------------------
               !! chlorophyll
               bchn = b0 * ( &
                 + ((frn * fprn * zphn) - fgmipn - fgmepn - fdpn - fdpn2) * (fthetan / xxi) )
               bchd = b0 * ( &
                 + ((frd * fprd * zphd) - fgmepd - fdpd - fdpd2) * (fthetad / xxi) )
               !!
               !!----------------------------------------------------------------------
               !! phytoplankton
               bphn = b0 * ( &
                 + (fprn * zphn) - fgmipn - fgmepn - fdpn - fdpn2 )
               bphd = b0 * ( &
                 + (fprd * zphd) - fgmepd - fdpd - fdpd2 )
               bpds = b0 * ( &
                 + (fprds * zpds) - fgmepds - fdpds - fsdiss - fdpds2 )	 
               !!
               !!----------------------------------------------------------------------
               !! zooplankton
               bzmi = b0 * ( &
                 + fmigrow - fgmezmi - fdzmi - fdzmi2 )
               bzme = b0 * ( &
                 + fmegrow - fdzme - fdzme2 )
               !!
               !!----------------------------------------------------------------------
               !! detritus
               bdet = b0 * ( &
                 + fdpn + ((1.0 - xfdfrac1) * fdpd)              &  ! mort. losses
                 + fdzmi + ((1.0 - xfdfrac2) * fdzme)            &  ! mort. losses
                 + ((1.0 - xbetan) * (finmi + finme))            &  ! assim. inefficiency
                 - fgmid - fgmed - fdd                           &  ! grazing and remin.
                 + ffast2slown )                                    ! seafloor fast->slow
               !!
               !!----------------------------------------------------------------------
               !! dissolved inorganic nitrogen nutrient
               fn_cons = 0.0  &
                 - (fprn * zphn) - (fprd * zphd)                    ! primary production
               fn_prod = 0.0  &
                 + (xphi * (fgmipn + fgmid))                     &  ! messy feeding remin.
                 + (xphi * (fgmepn + fgmepd + fgmezmi + fgmed))  &  ! messy feeding remin.
                 + fmiexcr + fmeexcr + fdd + freminn             &  ! excretion and remin.
		 + fdpn2 + fdpd2 + fdzmi2 + fdzme2                  ! metab. losses
               !! 
               !! riverine flux
               if ( jriver_n .gt. 0 ) then
                  f_riv_loc_n = f_riv_n(ji,jj) * friver_dep(jk,(mbathy(ji,jj)-1)) / fthk
                  fn_prod = fn_prod + f_riv_loc_n
               endif
               !!  
               !! benthic remineralisation
               if (jk.eq.(mbathy(ji,jj)-1) .and. jorgben.eq.1 .and. ibenthic.eq.1) then
                  fn_prod = fn_prod + (f_benout_n(ji,jj) / fthk)
               endif
               !!
               bdin = b0 * ( &
                 fn_prod + fn_cons )
               !!
               fnit_cons(ji,jj) = fnit_cons(ji,jj) + ( fthk * (  &  ! consumption of dissolved nitrogen
                 fn_cons ) )
               fnit_prod(ji,jj) = fnit_prod(ji,jj) + ( fthk * (  &  ! production of dissolved nitrogen
                 fn_prod ) )
               !!
               !!----------------------------------------------------------------------
               !! dissolved silicic acid nutrient
               fs_cons = 0.0  &
                 - (fprds * zpds)                                   ! opal production
               fs_prod = 0.0  &
                 + fsdiss                                        &  ! opal dissolution
                 + ((1.0 - xfdfrac1) * fdpds)                    &  ! mort. loss
                 + ((1.0 - xfdfrac3) * fgmepds)                  &  ! egestion of grazed Si
                 + freminsi + fdpds2                                ! fast diss. and metab. losses
               !! 
               !! riverine flux
               if ( jriver_si .gt. 0 ) then
                  f_riv_loc_si = f_riv_si(ji,jj) * friver_dep(jk,(mbathy(ji,jj)-1)) / fthk
                  fs_prod = fs_prod + f_riv_loc_si
               endif
               !!  
               !! benthic remineralisation
               if (jk.eq.(mbathy(ji,jj)-1) .and. jinorgben.eq.1 .and. ibenthic.eq.1) then
                  fs_prod = fs_prod + (f_benout_si(ji,jj) / fthk)
               endif
               !!
               bsil = b0 * ( &
                 fs_prod + fs_cons )
               !!
               fsil_cons(ji,jj) = fsil_cons(ji,jj) + ( fthk * (  &  ! consumption of dissolved silicon
                 fs_cons ) )
               fsil_prod(ji,jj) = fsil_prod(ji,jj) + ( fthk * (  &  ! production of dissolved silicon
                 fs_prod ) )
               !!
               !!----------------------------------------------------------------------
               !! dissolved "iron" nutrient
               bfer = b0 * ( &
               + (xrfn * bdin) + ffetop + ffebot - ffescav )

#if defined key_roam
               !!
               !!----------------------------------------------------------------------
               !! AXY (26/11/08): implicit detrital carbon change
               bdtc = b0 * ( &
                 + (xthetapn * fdpn) + ((1.0 - xfdfrac1) * (xthetapd * fdpd))      &  ! mort. losses
                 + (xthetazmi * fdzmi) + ((1.0 - xfdfrac2) * (xthetazme * fdzme))  &  ! mort. losses
                 + ((1.0 - xbetac) * (ficmi + ficme))                              &  ! assim. inefficiency
                 - fgmidc - fgmedc - fddc                                          &  ! grazing and remin.
                 + ffast2slowc )                                                      ! seafloor fast->slow
               !!
               !!----------------------------------------------------------------------
               !! dissolved inorganic carbon
               fc_cons = 0.0  &
                 - (xthetapn * fprn * zphn) - (xthetapd * fprd * zphd)                ! primary production
               fc_prod = 0.0  &
                 + (xthetapn * xphi * fgmipn) + (xphi * fgmidc)                    &  ! messy feeding remin
                 + (xthetapn * xphi * fgmepn) + (xthetapd * xphi * fgmepd)         &  ! messy feeding remin
                 + (xthetazmi * xphi * fgmezmi) + (xphi * fgmedc)                  &  ! messy feeding remin
                 + fmiresp + fmeresp + fddc + freminc + (xthetapn * fdpn2)         &  ! resp., remin., losses
                 + (xthetapd * fdpd2) + (xthetazmi * fdzmi2)                       &  ! losses
                 + (xthetazme * fdzme2) - ftempca + freminca                          ! losses, CaCO3
               !! 
               !! riverine flux
               if ( jriver_c .gt. 0 ) then
                  f_riv_loc_c = f_riv_c(ji,jj) * friver_dep(jk,(mbathy(ji,jj)-1)) / fthk
                  fc_prod = fc_prod + f_riv_loc_c
               endif
               !!  
               !! benthic remineralisation
               if (jk.eq.(mbathy(ji,jj)-1) .and. jorgben.eq.1 .and. ibenthic.eq.1) then
                  fc_prod = fc_prod + (f_benout_c(ji,jj) / fthk)
               endif
               if (jk.eq.(mbathy(ji,jj)-1) .and. jinorgben.eq.1 .and. ibenthic.eq.1) then
                  fc_prod = fc_prod + (f_benout_ca(ji,jj) / fthk)
               endif
               !!
               bdic = b0 * ( &
                 fc_prod + fc_cons )
               !!
               fcar_cons(ji,jj) = fcar_cons(ji,jj) + ( fthk * (  &  ! consumption of dissolved carbon
                 fc_cons ) )
               fcar_prod(ji,jj) = fcar_prod(ji,jj) + ( fthk * (  &  ! production of dissolved carbon
                 fc_prod ) )
               !!
               !!----------------------------------------------------------------------
               !! alkalinity
               fa_prod = 0.0  &
                 + (2.0 * freminca)                                                   ! CaCO3 dissolution
               fa_cons = 0.0  &
                 - (2.0 * ftempca)                                                    ! CaCO3 production
               !! 
               !! riverine flux
               if ( jriver_alk .gt. 0 ) then
                  f_riv_loc_alk = f_riv_alk(ji,jj) * friver_dep(jk,(mbathy(ji,jj)-1)) / fthk
                  fa_prod = fa_prod + f_riv_loc_alk
               endif
               !!  
               !! benthic remineralisation
               if (jk.eq.(mbathy(ji,jj)-1) .and. jinorgben.eq.1 .and. ibenthic.eq.1) then
                  fa_prod = fa_prod + (2.0 * f_benout_ca(ji,jj) / fthk)
               endif
               !!
               balk = b0 * ( &
                 fa_prod + fa_cons )
               !!
               !!----------------------------------------------------------------------
               !! oxygen (has protection at low O2 concentrations; OCMIP-2 style)
               fo2_prod = 0.0 &
                 + (xthetanit * fprn * zphn)                                      & ! Pn primary production, N
                 + (xthetanit * fprd * zphd)                                      & ! Pd primary production, N
                 + (xthetarem * xthetapn * fprn * zphn)                           & ! Pn primary production, C
                 + (xthetarem * xthetapd * fprd * zphd)                             ! Pd primary production, C
               fo2_ncons = 0.0 &
                 - (xthetanit * xphi * fgmipn)                                    & ! Pn messy feeding remin., N
                 - (xthetanit * xphi * fgmid)                                     & ! D  messy feeding remin., N
                 - (xthetanit * xphi * fgmepn)                                    & ! Pn messy feeding remin., N
                 - (xthetanit * xphi * fgmepd)                                    & ! Pd messy feeding remin., N
                 - (xthetanit * xphi * fgmezmi)                                   & ! Zi messy feeding remin., N
                 - (xthetanit * xphi * fgmed)                                     & ! D  messy feeding remin., N
                 - (xthetanit * fmiexcr)                                          & ! microzoo excretion, N
                 - (xthetanit * fmeexcr)                                          & ! mesozoo  excretion, N
                 - (xthetanit * fdd)                                              & ! slow detritus remin., N 
                 - (xthetanit * freminn)                                          & ! fast detritus remin., N
                 - (xthetanit * fdpn2)                                            & ! Pn  losses, N
                 - (xthetanit * fdpd2)                                            & ! Pd  losses, N
                 - (xthetanit * fdzmi2)                                           & ! Zmi losses, N
                 - (xthetanit * fdzme2)                                             ! Zme losses, N
               !!  
               !! benthic remineralisation
               if (jk.eq.(mbathy(ji,jj)-1) .and. jorgben.eq.1 .and. ibenthic.eq.1) then
                  fo2_ncons = fo2_ncons - (xthetanit * f_benout_n(ji,jj) / fthk)
               endif
               fo2_ccons = 0.0 &
                 - (xthetarem * xthetapn * xphi * fgmipn)                         & ! Pn messy feeding remin., C
                 - (xthetarem * xphi * fgmidc)                                    & ! D  messy feeding remin., C
                 - (xthetarem * xthetapn * xphi * fgmepn)                         & ! Pn messy feeding remin., C
                 - (xthetarem * xthetapd * xphi * fgmepd)                         & ! Pd messy feeding remin., C
                 - (xthetarem * xthetazmi * xphi * fgmezmi)                       & ! Zi messy feeding remin., C
                 - (xthetarem * xphi * fgmedc)                                    & ! D  messy feeding remin., C
                 - (xthetarem * fmiresp)                                          & ! microzoo respiration, C
                 - (xthetarem * fmeresp)                                          & ! mesozoo  respiration, C
                 - (xthetarem * fddc)                                             & ! slow detritus remin., C
                 - (xthetarem * freminc)                                          & ! fast detritus remin., C
                 - (xthetarem * xthetapn * fdpn2)                                 & ! Pn  losses, C
                 - (xthetarem * xthetapd * fdpd2)                                 & ! Pd  losses, C
                 - (xthetarem * xthetazmi * fdzmi2)                               & ! Zmi losses, C
                 - (xthetarem * xthetazme * fdzme2)                                 ! Zme losses, C
               !!  
               !! benthic remineralisation
               if (jk.eq.(mbathy(ji,jj)-1) .and. jorgben.eq.1 .and. ibenthic.eq.1) then
                  fo2_ccons = fo2_ccons - (xthetarem * f_benout_c(ji,jj) / fthk)
               endif
               fo2_cons = fo2_ncons + fo2_ccons
               !!
               !! is this a suboxic zone?
               if (zoxy.lt.xo2min) then  ! deficient O2; production fluxes only
                  boxy = b0 * ( &
                    fo2_prod )
                  foxy_prod(ji,jj) = foxy_prod(ji,jj) + ( fthk * fo2_prod )
                  foxy_anox(ji,jj) = foxy_anox(ji,jj) + ( fthk * fo2_cons )
               else                      ! sufficient O2; production + consumption fluxes
                  boxy = b0 * ( &
                    fo2_prod + fo2_cons )
                  foxy_prod(ji,jj) = foxy_prod(ji,jj) + ( fthk * fo2_prod )
                  foxy_cons(ji,jj) = foxy_cons(ji,jj) + ( fthk * fo2_cons )
               endif
               !!
               !! air-sea fluxes (if this is the surface box)
               if (jk.eq.1) then
                  !!
                  !! CO2 flux
                  bdic = bdic + (b0 * f_co2flux)
                  !!
                  !! O2 flux (mol/m3/s -> mmol/m3/d)
                  boxy = boxy + (b0 * f_o2flux)
               endif
#endif

	       !! report state variable fluxes (not including fast-sinking detritus)
               if (idf.eq.1.AND.idfval.eq.1) then
                  IF (lwp) write (numout,*) '------------------------------'
                  IF (lwp) write (numout,*) 'bchn(',jk,')  = ', bchn
                  IF (lwp) write (numout,*) 'bchd(',jk,')  = ', bchd
                  IF (lwp) write (numout,*) 'bphn(',jk,')  = ', bphn
                  IF (lwp) write (numout,*) 'bphd(',jk,')  = ', bphd
                  IF (lwp) write (numout,*) 'bpds(',jk,')  = ', bpds
                  IF (lwp) write (numout,*) 'bzmi(',jk,')  = ', bzmi
                  IF (lwp) write (numout,*) 'bzme(',jk,')  = ', bzme
                  IF (lwp) write (numout,*) 'bdet(',jk,')  = ', bdet
                  IF (lwp) write (numout,*) 'bdin(',jk,')  = ', bdin
                  IF (lwp) write (numout,*) 'bsil(',jk,')  = ', bsil
                  IF (lwp) write (numout,*) 'bfer(',jk,')  = ', bfer
#if defined key_roam
                  IF (lwp) write (numout,*) 'bdtc(',jk,')  = ', bdtc
                  IF (lwp) write (numout,*) 'bdic(',jk,')  = ', bdic
                  IF (lwp) write (numout,*) 'balk(',jk,')  = ', balk
                  IF (lwp) write (numout,*) 'boxy(',jk,')  = ', boxy
#endif
               endif

               !!----------------------------------------------------------------------
               !! Integrate calculated fluxes for mass balance
               !!----------------------------------------------------------------------
               !!
               !! === nitrogen ===
               fflx_n(ji,jj)  = fflx_n(ji,jj)  + &
                  fthk * ( bphn + bphd + bzmi + bzme + bdet + bdin )
               !! === silicon ===
               fflx_si(ji,jj) = fflx_si(ji,jj) + &
                  fthk * ( bpds + bsil )
               !! === iron ===
               fflx_fe(ji,jj) = fflx_fe(ji,jj) + &
                  fthk * ( ( xrfn * ( bphn + bphd + bzmi + bzme + bdet) ) + bfer )
#if defined key_roam
               !! === carbon ===
               fflx_c(ji,jj)  = fflx_c(ji,jj)  + &
                  fthk * ( (xthetapn * bphn) + (xthetapd * bphd) + &
                  (xthetazmi * bzmi) + (xthetazme * bzme) + bdtc + bdic )
               !! === alkalinity ===
               fflx_a(ji,jj)  = fflx_a(ji,jj)  + &
                  fthk * ( balk )
               !! === oxygen ===
               fflx_o2(ji,jj) = fflx_o2(ji,jj) + &
                  fthk * ( boxy )
#endif

               !!----------------------------------------------------------------------
               !! Apply calculated tracer fluxes
               !!----------------------------------------------------------------------
               !!
               !! units: [unit of tracer] per second (fluxes are calculated above per day)
               !!
	       ibio_switch = 1
	       if (ibio_switch.eq.1) then
               tra(ji,jj,jk,jpchn) = tra(ji,jj,jk,jpchn) + (bchn / 86400.)
               tra(ji,jj,jk,jpchd) = tra(ji,jj,jk,jpchd) + (bchd / 86400.)
               tra(ji,jj,jk,jpphn) = tra(ji,jj,jk,jpphn) + (bphn / 86400.)
               tra(ji,jj,jk,jpphd) = tra(ji,jj,jk,jpphd) + (bphd / 86400.)
               tra(ji,jj,jk,jppds) = tra(ji,jj,jk,jppds) + (bpds / 86400.)
               tra(ji,jj,jk,jpzmi) = tra(ji,jj,jk,jpzmi) + (bzmi / 86400.)
               tra(ji,jj,jk,jpzme) = tra(ji,jj,jk,jpzme) + (bzme / 86400.)
               tra(ji,jj,jk,jpdet) = tra(ji,jj,jk,jpdet) + (bdet / 86400.)
               tra(ji,jj,jk,jpdin) = tra(ji,jj,jk,jpdin) + (bdin / 86400.)
               tra(ji,jj,jk,jpsil) = tra(ji,jj,jk,jpsil) + (bsil / 86400.)
               tra(ji,jj,jk,jpfer) = tra(ji,jj,jk,jpfer) + (bfer / 86400.)
#if defined key_roam
               tra(ji,jj,jk,jpdtc) = tra(ji,jj,jk,jpdtc) + (bdtc / 86400.)
               tra(ji,jj,jk,jpdic) = tra(ji,jj,jk,jpdic) + (bdic / 86400.)
               tra(ji,jj,jk,jpalk) = tra(ji,jj,jk,jpalk) + (balk / 86400.)
               tra(ji,jj,jk,jpoxy) = tra(ji,jj,jk,jpoxy) + (boxy / 86400.)
#endif
	       endif               

               !!----------------------------------------------------------------------
               !! Check model conservation
               !! these terms merely sum up the tendency terms of the relevant
               !! state variables, which should sum to zero; the iron cycle is
               !! complicated by fluxes that add (aeolian deposition and seafloor
               !! remineralisation) and remove (scavenging) dissolved iron from
               !! the model (i.e. the sum of iron fluxes is unlikely to be zero)
               !!----------------------------------------------------------------------
               !!
               fnit0 = bphn + bphd + bzme + bzmi + bdet + bdin  ! + ftempn
               fsil0 = bpds + bsil                              ! + ftempsi
               ffer0 = (xrfn * fnit0) + bfer
#if defined key_roam
               fcar0 = 0.
               falk0 = 0.
               foxy0 = 0.
#endif
               !!
               if (kt/240*240.eq.kt) then
                  if (ji.eq.2.and.jj.eq.2.and.jk.eq.1) then
                     IF (lwp) write (*,*) '*******!MEDUSA Conservation!*******',kt
#if defined key_roam
                     IF (lwp) write (*,*) fnit0,fsil0,ffer0,fcar0,falk0,foxy0
#else
                     IF (lwp) write (*,*) fnit0,fsil0,ffer0
#endif
                  endif
               endif     

#if defined key_trc_diabio
               !!======================================================================
               !! LOCAL GRID CELL DIAGNOSTICS
               !!======================================================================
               !!
               !!----------------------------------------------------------------------
               !! Full diagnostics key_trc_diabio:
               !! LOBSTER and PISCES support full diagnistics option key_trc_diabio    
               !! which gives an option of FULL output of biological sourses and sinks.
               !! I cannot see any reason for doing this. If needed, it can be done
               !! as shown below.
               !!----------------------------------------------------------------------
               !!
               IF(lwp) WRITE(numout,*) ' MEDUSA does not support key_trc_diabio'
               !!               trbio(ji,jj,jk, 1) = fprn
#endif

#if defined key_trc_diaadd
               !!----------------------------------------------------------------------
               !! Prepare 2D diagnostics
               !!----------------------------------------------------------------------
               !!
               !! if ((kt / 240*240).eq.kt) then
               !!    IF (lwp) write (*,*) '*******!MEDUSA DIAADD!*******',kt
               !! endif     
               trc2d(ji,jj,1)  = trc2d(ji,jj,1)  + ftot_n                   !! nitrogen inventory
               trc2d(ji,jj,2)  = trc2d(ji,jj,2)  + ftot_si                  !! silicon  inventory
               trc2d(ji,jj,3)  = trc2d(ji,jj,3)  + ftot_fe                  !! iron     inventory
               trc2d(ji,jj,4)  = trc2d(ji,jj,4)  + (fprn  * zphn * fthk)    !! non-diatom production
               trc2d(ji,jj,5)  = trc2d(ji,jj,5)  + (fdpn         * fthk)    !! non-diatom non-grazing losses
               trc2d(ji,jj,6)  = trc2d(ji,jj,6)  + (fprd  * zphd * fthk)    !! diatom production
               trc2d(ji,jj,7)  = trc2d(ji,jj,7)  + (fdpd         * fthk)    !! diatom non-grazing losses
               !! diagnostic field  8 is (ostensibly) supplied by trcsed.F            
               trc2d(ji,jj,9)  = trc2d(ji,jj,9)  + (fprds * zpds * fthk)    !! diatom silicon production
               trc2d(ji,jj,10) = trc2d(ji,jj,10) + (fsdiss  * fthk)         !! diatom silicon dissolution
               trc2d(ji,jj,11) = trc2d(ji,jj,11) + (fgmipn  * fthk)         !! microzoo grazing on non-diatoms
               trc2d(ji,jj,12) = trc2d(ji,jj,12) + (fgmid   * fthk)         !! microzoo grazing on detrital nitrogen
               trc2d(ji,jj,13) = trc2d(ji,jj,13) + (fdzmi   * fthk)         !! microzoo non-grazing losses
               trc2d(ji,jj,14) = trc2d(ji,jj,14) + (fgmepn  * fthk)         !! mesozoo  grazing on non-diatoms
               trc2d(ji,jj,15) = trc2d(ji,jj,15) + (fgmepd  * fthk)         !! mesozoo  grazing on diatoms
               trc2d(ji,jj,16) = trc2d(ji,jj,16) + (fgmezmi * fthk)         !! mesozoo  grazing on microzoo
               trc2d(ji,jj,17) = trc2d(ji,jj,17) + (fgmed   * fthk)         !! mesozoo  grazing on detrital nitrogen
               trc2d(ji,jj,18) = trc2d(ji,jj,18) + (fdzme   * fthk)         !! mesozoo  non-grazing losses
               !! diagnostic field 19 is (ostensibly) supplied by trcexp.F
               trc2d(ji,jj,20) = trc2d(ji,jj,20) + (fslown  * fthk)         !! slow sinking detritus N production
               trc2d(ji,jj,21) = trc2d(ji,jj,21) + (fdd     * fthk)         !! detrital remineralisation
               trc2d(ji,jj,22) = trc2d(ji,jj,22) + (ffetop  * fthk)         !! aeolian  iron addition
               trc2d(ji,jj,23) = trc2d(ji,jj,23) + (ffebot  * fthk)         !! seafloor iron addition
               trc2d(ji,jj,24) = trc2d(ji,jj,24) + (ffescav * fthk)         !! "free" iron scavenging
	       trc2d(ji,jj,25) = trc2d(ji,jj,25) + (fjln  * zphn * fthk)    !! non-diatom J  limitation term 
	       trc2d(ji,jj,26) = trc2d(ji,jj,26) + (fnln  * zphn * fthk)    !! non-diatom N  limitation term 
	       trc2d(ji,jj,27) = trc2d(ji,jj,27) + (ffln  * zphn * fthk)    !! non-diatom Fe limitation term 
	       trc2d(ji,jj,28) = trc2d(ji,jj,28) + (fjld  * zphd * fthk)    !! diatom     J  limitation term 
	       trc2d(ji,jj,29) = trc2d(ji,jj,29) + (fnld  * zphd * fthk)    !! diatom     N  limitation term 
	       trc2d(ji,jj,30) = trc2d(ji,jj,30) + (ffld  * zphd * fthk)    !! diatom     Fe limitation term 
	       trc2d(ji,jj,31) = trc2d(ji,jj,31) + (fsld2 * zphd * fthk)    !! diatom     Si limitation term 
	       trc2d(ji,jj,32) = trc2d(ji,jj,32) + (fsld  * zphd * fthk)    !! diatom     Si uptake limitation term
               if (jk.eq.i0100) trc2d(ji,jj,33) = fslownflux                !! slow detritus flux at  100 m
               if (jk.eq.i0200) trc2d(ji,jj,34) = fslownflux                !! slow detritus flux at  200 m
               if (jk.eq.i0500) trc2d(ji,jj,35) = fslownflux                !! slow detritus flux at  500 m
               if (jk.eq.i1000) trc2d(ji,jj,36) = fslownflux                !! slow detritus flux at 1000 m
               trc2d(ji,jj,37) = trc2d(ji,jj,37) + fregen                   !! non-fast N  full column regeneration
               trc2d(ji,jj,38) = trc2d(ji,jj,38) + fregensi                 !! non-fast Si full column regeneration
               if (jk.eq.i0100) trc2d(ji,jj,39) = trc2d(ji,jj,37)           !! non-fast N  regeneration to  100 m
               if (jk.eq.i0200) trc2d(ji,jj,40) = trc2d(ji,jj,37)           !! non-fast N  regeneration to  200 m
               if (jk.eq.i0500) trc2d(ji,jj,41) = trc2d(ji,jj,37)           !! non-fast N  regeneration to  500 m
               if (jk.eq.i1000) trc2d(ji,jj,42) = trc2d(ji,jj,37)           !! non-fast N  regeneration to 1000 m
               trc2d(ji,jj,43) = trc2d(ji,jj,43) + (ftempn  * fthk)         !! fast sinking detritus N production
               trc2d(ji,jj,44) = trc2d(ji,jj,44) + (ftempsi * fthk)         !! fast sinking detritus Si production
               trc2d(ji,jj,45) = trc2d(ji,jj,45) + (ftempfe * fthk)         !! fast sinking detritus Fe production
               trc2d(ji,jj,46) = trc2d(ji,jj,46) + (ftempc  * fthk)         !! fast sinking detritus C production
               trc2d(ji,jj,47) = trc2d(ji,jj,47) + (ftempca * fthk)         !! fast sinking detritus CaCO3 production
               if (jk.eq.i0100) trc2d(ji,jj,48) = ffastn(ji,jj)             !! fast detritus N  flux at  100 m
               if (jk.eq.i0200) trc2d(ji,jj,49) = ffastn(ji,jj)             !! fast detritus N  flux at  200 m
               if (jk.eq.i0500) trc2d(ji,jj,50) = ffastn(ji,jj)             !! fast detritus N  flux at  500 m
               if (jk.eq.i1000) trc2d(ji,jj,51) = ffastn(ji,jj)             !! fast detritus N  flux at 1000 m
               if (jk.eq.i0100) trc2d(ji,jj,52) = fregenfast(ji,jj)         !! N  regeneration to  100 m
               if (jk.eq.i0200) trc2d(ji,jj,53) = fregenfast(ji,jj)         !! N  regeneration to  200 m
               if (jk.eq.i0500) trc2d(ji,jj,54) = fregenfast(ji,jj)         !! N  regeneration to  500 m
               if (jk.eq.i1000) trc2d(ji,jj,55) = fregenfast(ji,jj)         !! N  regeneration to 1000 m
               if (jk.eq.i0100) trc2d(ji,jj,56) = ffastsi(ji,jj)            !! fast detritus Si flux at  100 m
               if (jk.eq.i0200) trc2d(ji,jj,57) = ffastsi(ji,jj)            !! fast detritus Si flux at  200 m
               if (jk.eq.i0500) trc2d(ji,jj,58) = ffastsi(ji,jj)            !! fast detritus Si flux at  500 m
               if (jk.eq.i1000) trc2d(ji,jj,59) = ffastsi(ji,jj)            !! fast detritus Si flux at 1000 m
               if (jk.eq.i0100) trc2d(ji,jj,60) = fregenfastsi(ji,jj)       !! Si regeneration to  100 m
               if (jk.eq.i0200) trc2d(ji,jj,61) = fregenfastsi(ji,jj)       !! Si regeneration to  200 m
               if (jk.eq.i0500) trc2d(ji,jj,62) = fregenfastsi(ji,jj)       !! Si regeneration to  500 m
               if (jk.eq.i1000) trc2d(ji,jj,63) = fregenfastsi(ji,jj)       !! Si regeneration to 1000 m
               trc2d(ji,jj,64) = trc2d(ji,jj,64) + (freminn  * fthk)        !! sum of fast-sinking N  fluxes
               trc2d(ji,jj,65) = trc2d(ji,jj,65) + (freminsi * fthk)        !! sum of fast-sinking Si fluxes
               trc2d(ji,jj,66) = trc2d(ji,jj,66) + (freminfe * fthk)        !! sum of fast-sinking Fe fluxes
               trc2d(ji,jj,67) = trc2d(ji,jj,67) + (freminc  * fthk)        !! sum of fast-sinking C  fluxes
               trc2d(ji,jj,68) = trc2d(ji,jj,68) + (freminca * fthk)        !! sum of fast-sinking Ca fluxes
               if (jk.eq.(mbathy(ji,jj)-1)) then
                  trc2d(ji,jj,69) = fsedn                                   !! N  sedimentation flux                                  
                  trc2d(ji,jj,70) = fsedsi                                  !! Si sedimentation flux
                  trc2d(ji,jj,71) = fsedfe                                  !! Fe sedimentation flux
                  trc2d(ji,jj,72) = fsedc                                   !! C  sedimentation flux
                  trc2d(ji,jj,73) = fsedca                                  !! Ca sedimentation flux
               endif
	       if (jk.eq.1)  trc2d(ji,jj,74) = qsr(ji,jj)
	       if (jk.eq.1)  trc2d(ji,jj,75) = xpar(ji,jj,jk)
               !! diagnostic fields 76 to 80 calculated below
               trc2d(ji,jj,81) = trc2d(ji,jj,81) + fprn_ml                  !! mixed layer non-diatom production
               trc2d(ji,jj,82) = trc2d(ji,jj,82) + fprd_ml                  !! mixed layer     diatom production
               trc2d(ji,jj,83) = ocal_ccd(ji,jj)                            !! calcite CCD depth
               trc2d(ji,jj,84) = fccd(ji,jj)                                !! last model level above calcite CCD depth
               if (jk.eq.1)     trc2d(ji,jj,85) = xFree                     !! surface "free" iron
               if (jk.eq.i0200) trc2d(ji,jj,86) = xFree                     !! "free" iron at  100 m
               if (jk.eq.i0200) trc2d(ji,jj,87) = xFree                     !! "free" iron at  200 m
               if (jk.eq.i0500) trc2d(ji,jj,88) = xFree                     !! "free" iron at  500 m
               if (jk.eq.i1000) trc2d(ji,jj,89) = xFree                     !! "free" iron at 1000 m
               !! AXY (27/06/12): extract "euphotic depth"
               if (jk.eq.1)     trc2d(ji,jj,90) = xze(ji,jj)
               !! 
               ftot_pn(ji,jj)  = ftot_pn(ji,jj)  + (zphn * fthk)            !! vertical integral non-diatom phytoplankton
               ftot_pd(ji,jj)  = ftot_pd(ji,jj)  + (zphd * fthk)            !! vertical integral diatom phytoplankton

#if defined key_roam
               !! ROAM provisionally has access to a further 20 2D diagnostics
               if (jk .eq. 1) then
                  trc2d(ji,jj,91)  = trc2d(ji,jj,91)  + f_wind              !! surface wind
                  trc2d(ji,jj,92)  = trc2d(ji,jj,92)  + f_pco2a             !! atmospheric pCO2
                  trc2d(ji,jj,93)  = trc2d(ji,jj,93)  + f_ph                !! ocean pH
                  trc2d(ji,jj,94)  = trc2d(ji,jj,94)  + f_pco2w             !! ocean pCO2
                  trc2d(ji,jj,95)  = trc2d(ji,jj,95)  + f_h2co3             !! ocean H2CO3 conc.
                  trc2d(ji,jj,96)  = trc2d(ji,jj,96)  + f_hco3              !! ocean HCO3 conc.
                  trc2d(ji,jj,97)  = trc2d(ji,jj,97)  + f_co3               !! ocean CO3 conc.
                  trc2d(ji,jj,98)  = trc2d(ji,jj,98)  + f_co2flux           !! air-sea CO2 flux
                  trc2d(ji,jj,99)  = trc2d(ji,jj,99)  + f_omcal(ji,jj)      !! ocean omega calcite 
                  trc2d(ji,jj,100) = trc2d(ji,jj,100) + f_omarg(ji,jj)      !! ocean omega aragonite
                  trc2d(ji,jj,101) = trc2d(ji,jj,101) + f_TDIC              !! ocean TDIC
                  trc2d(ji,jj,102) = trc2d(ji,jj,102) + f_TALK              !! ocean TALK
                  trc2d(ji,jj,103) = trc2d(ji,jj,103) + f_kw660             !! surface kw660
                  trc2d(ji,jj,104) = trc2d(ji,jj,104) + f_pp0               !! surface pressure
                  trc2d(ji,jj,105) = trc2d(ji,jj,105) + f_o2flux            !! air-sea O2 flux
                  trc2d(ji,jj,106) = trc2d(ji,jj,106) + f_o2sat             !! ocean O2 saturation
                  trc2d(ji,jj,107) = f2_ccd_cal(ji,jj)                      !! depth calcite CCD
                  trc2d(ji,jj,108) = f2_ccd_arg(ji,jj)                      !! depth aragonite CCD
               endif
               if (jk .eq. (mbathy(ji,jj)-1)) then
                  trc2d(ji,jj,109) = f3_omcal(ji,jj,jk)                     !! seafloor omega calcite
                  trc2d(ji,jj,110) = f3_omarg(ji,jj,jk)                     !! seafloor omega aragonite
               endif
               !! diagnostic fields 111 to 117 calculated below
               if (jk.eq.i0100) trc2d(ji,jj,118) = ffastca(ji,jj)/ffastc(ji,jj)  !! rain ratio at  100 m
               if (jk.eq.i0500) trc2d(ji,jj,119) = ffastca(ji,jj)/ffastc(ji,jj)  !! rain ratio at  500 m
               if (jk.eq.i1000) trc2d(ji,jj,120) = ffastca(ji,jj)/ffastc(ji,jj)  !! rain ratio at 1000 m
               !! AXY (18/01/12): benthic flux diagnostics
               if (jk.eq.(mbathy(ji,jj)-1)) then
                  trc2d(ji,jj,121) = f_sbenin_n(ji,jj)  + f_fbenin_n(ji,jj)
                  trc2d(ji,jj,122) = f_sbenin_fe(ji,jj) + f_fbenin_fe(ji,jj)
                  trc2d(ji,jj,123) = f_sbenin_c(ji,jj)  + f_fbenin_c(ji,jj)
                  trc2d(ji,jj,124) = f_fbenin_si(ji,jj)
                  trc2d(ji,jj,125) = f_fbenin_ca(ji,jj)
                  trc2d(ji,jj,126) = f_benout_n(ji,jj)
                  trc2d(ji,jj,127) = f_benout_fe(ji,jj)
                  trc2d(ji,jj,128) = f_benout_c(ji,jj)
                  trc2d(ji,jj,129) = f_benout_si(ji,jj)
                  trc2d(ji,jj,130) = f_benout_ca(ji,jj)
               endif
               !! diagnostics fields 131 to 135 calculated below
               trc2d(ji,jj,136) = f_runoff(ji,jj)
               !! AXY (19/07/12): amended to allow for riverine nutrient addition below surface
               trc2d(ji,jj,137) = trc2d(ji,jj,137) + (f_riv_loc_n * fthk)
               trc2d(ji,jj,138) = trc2d(ji,jj,138) + (f_riv_loc_si * fthk)
               trc2d(ji,jj,139) = trc2d(ji,jj,139) + (f_riv_loc_c * fthk)
               trc2d(ji,jj,140) = trc2d(ji,jj,140) + (f_riv_loc_alk * fthk)
               trc2d(ji,jj,141) = trc2d(ji,jj,141) + (fslowc  * fthk)       !! slow sinking detritus C production
               if (jk.eq.i0100) trc2d(ji,jj,142) = fslowcflux               !! slow detritus flux at  100 m
               if (jk.eq.i0200) trc2d(ji,jj,143) = fslowcflux               !! slow detritus flux at  200 m
               if (jk.eq.i0500) trc2d(ji,jj,144) = fslowcflux               !! slow detritus flux at  500 m
               if (jk.eq.i1000) trc2d(ji,jj,145) = fslowcflux               !! slow detritus flux at 1000 m
               trc2d(ji,jj,146)  = trc2d(ji,jj,146)  + ftot_c               !! carbon     inventory
               trc2d(ji,jj,147)  = trc2d(ji,jj,147)  + ftot_a               !! alkalinity inventory
               trc2d(ji,jj,148)  = trc2d(ji,jj,148)  + ftot_o2              !! oxygen     inventory
               if (jk.eq.(mbathy(ji,jj)-1)) then
                  trc2d(ji,jj,149) = f_benout_lyso_ca(ji,jj)
               endif
#endif
               !! other possible future diagnostics include:
               !!   - integrated tracer values (esp. biological)
               !!   - mixed layer tracer values
               !!   - sub-surface chlorophyll maxima (plus depth)
               !!   - different mixed layer depth criteria (T, sigma, var. sigma)

               !!----------------------------------------------------------------------
               !! Prepare 3D diagnostics
               !!----------------------------------------------------------------------
               !!
               trc3d(ji,jj,jk,1)  = ((fprn + fprd) * zphn)     !! primary production  
               trc3d(ji,jj,jk,2)  = fslownflux + ffastn(ji,jj) !! detrital flux
	       trc3d(ji,jj,jk,3)  = fregen + (freminn * fthk)  !! remineralisation
# if defined key_roam
	       trc3d(ji,jj,jk,4)  = f3_pH(ji,jj,jk)            !! pH
               trc3d(ji,jj,jk,5)  = f3_omcal(ji,jj,jk)         !! omega calcite
# else
               trc3d(ji,jj,jk,4)  = ffastsi(ji,jj)             !! fast Si flux
# endif
#endif
             !! CLOSE wet point IF..THEN loop
            endif
         !! CLOSE horizontal loops
         END DO
         END DO
      !! CLOSE vertical loop
      END DO

      !!----------------------------------------------------------------------
      !! Process benthic in/out fluxes
      !! These can be handled outside of the 3D calculations since the
      !! benthic pools (and fluxes) are 2D in nature; this code is
      !! (shamelessly) borrowed from corresponding code in the LOBSTER
      !! model
      !!----------------------------------------------------------------------
      !!
      !! IF(lwp) WRITE(numout,*) 'AXY: rdt = ', rdt
      if (jorgben.eq.1) then
         za_sed_n(:,:)  = zn_sed_n(:,:)  + & 
         &                ( f_sbenin_n(:,:)  + f_fbenin_n(:,:)  - f_benout_n(:,:)  ) * (rdt / 86400.)
         zn_sed_n(:,:)  = za_sed_n(:,:)
         !!
         za_sed_fe(:,:) = zn_sed_fe(:,:) + &
         &                ( f_sbenin_fe(:,:) + f_fbenin_fe(:,:) - f_benout_fe(:,:) ) * (rdt / 86400.)
         zn_sed_fe(:,:) = za_sed_fe(:,:)
         !!
         za_sed_c(:,:)  = zn_sed_c(:,:)  + &
         &                ( f_sbenin_c(:,:)  + f_fbenin_c(:,:)  - f_benout_c(:,:)  ) * (rdt / 86400.)
         zn_sed_c(:,:)  = za_sed_c(:,:)
      endif
      if (jinorgben.eq.1) then
         za_sed_si(:,:) = zn_sed_si(:,:) + & 
         &                ( f_fbenin_si(:,:) - f_benout_si(:,:) ) * (rdt / 86400.)
         zn_sed_si(:,:) = za_sed_si(:,:)
         !!
         za_sed_ca(:,:) = zn_sed_ca(:,:) + &
         &                ( f_fbenin_ca(:,:) - f_benout_ca(:,:) ) * (rdt / 86400.)
         zn_sed_ca(:,:) = za_sed_ca(:,:)
      endif
      DO jj = 2,jpjm1
         DO ji = 2,jpim1
            trc2d(ji,jj,131) = za_sed_n(ji,jj)
            trc2d(ji,jj,132) = za_sed_fe(ji,jj)
            trc2d(ji,jj,133) = za_sed_c(ji,jj)
            trc2d(ji,jj,134) = za_sed_si(ji,jj)
            trc2d(ji,jj,135) = za_sed_ca(ji,jj)
         ENDDO
      ENDDO

      if (ibenthic.eq.2) then
         !! The code below (in this if ... then ... endif loop) is
         !! effectively commented out because it does not work as 
         !! anticipated; it can be deleted at a later date
      if (jorgben.eq.1) then
         za_sed_n(:,:)  = ( f_sbenin_n(:,:)  + f_fbenin_n(:,:)  - f_benout_n(:,:)  ) * rdt
         za_sed_fe(:,:) = ( f_sbenin_fe(:,:) + f_fbenin_fe(:,:) - f_benout_fe(:,:) ) * rdt
         za_sed_c(:,:)  = ( f_sbenin_c(:,:)  + f_fbenin_c(:,:)  - f_benout_c(:,:)  ) * rdt
      endif
      if (jinorgben.eq.1) then
         za_sed_si(:,:) = ( f_fbenin_si(:,:) - f_benout_si(:,:) ) * rdt
         za_sed_ca(:,:) = ( f_fbenin_ca(:,:) - f_benout_ca(:,:) ) * rdt
      endif
      !!
      !! Leap-frog scheme - only in explicit case, otherwise the time stepping
      !! is already being done in trczdf
      IF( l_trczdf_exp .AND. (ln_trcadv_cen2 .OR. ln_trcadv_tvd) ) THEN
         zfact = 2. * rdttra(jk) * FLOAT( ndttrc )
         IF( neuler == 0 .AND. kt == nittrc000 )   zfact = rdttra(jk) * FLOAT(ndttrc)
         if (jorgben.eq.1) then
            za_sed_n(:,:)  = zb_sed_n(:,:)  + ( zfact * za_sed_n(:,:)  )
            za_sed_fe(:,:) = zb_sed_fe(:,:) + ( zfact * za_sed_fe(:,:) )
            za_sed_c(:,:)  = zb_sed_c(:,:)  + ( zfact * za_sed_c(:,:)  )
         endif
         if (jinorgben.eq.1) then
            za_sed_si(:,:) = zb_sed_si(:,:) + ( zfact * za_sed_si(:,:) )
            za_sed_ca(:,:) = zb_sed_ca(:,:) + ( zfact * za_sed_ca(:,:) )
         endif
      ENDIF
      !! 
      !! Time filter and swap of arrays
      IF( ln_trcadv_cen2 .OR. ln_trcadv_tvd  ) THEN         ! centred or tvd scheme
         IF( neuler == 0 .AND. kt == nittrc000 ) THEN
            if (jorgben.eq.1) then
               zb_sed_n(:,:)  = zn_sed_n(:,:)
               zn_sed_n(:,:)  = za_sed_n(:,:)
               za_sed_n(:,:)  = 0.0
               !!
               zb_sed_fe(:,:) = zn_sed_fe(:,:)
               zn_sed_fe(:,:) = za_sed_fe(:,:)
               za_sed_fe(:,:) = 0.0
               !!
               zb_sed_c(:,:)  = zn_sed_c(:,:)
               zn_sed_c(:,:)  = za_sed_c(:,:)
               za_sed_c(:,:)  = 0.0
            endif
            if (jinorgben.eq.1) then
               zb_sed_si(:,:) = zn_sed_si(:,:)
               zn_sed_si(:,:) = za_sed_si(:,:)
               za_sed_si(:,:) = 0.0
               !!
               zb_sed_ca(:,:) = zn_sed_ca(:,:)
               zn_sed_ca(:,:) = za_sed_ca(:,:)
               za_sed_ca(:,:) = 0.0
            endif
         ELSE
            if (jorgben.eq.1) then
               zb_sed_n(:,:)  = (atfp  * ( zb_sed_n(:,:)  + za_sed_n(:,:)  )) + (atfp1 * zn_sed_n(:,:) )
               zn_sed_n(:,:)  = za_sed_n(:,:)
               za_sed_n(:,:)  = 0.0
               !!
               zb_sed_fe(:,:) = (atfp  * ( zb_sed_fe(:,:) + za_sed_fe(:,:) )) + (atfp1 * zn_sed_fe(:,:))
               zn_sed_fe(:,:) = za_sed_fe(:,:)
               za_sed_fe(:,:) = 0.0
               !!
               zb_sed_c(:,:)  = (atfp  * ( zb_sed_c(:,:)  + za_sed_c(:,:)  )) + (atfp1 * zn_sed_c(:,:) )
               zn_sed_c(:,:)  = za_sed_c(:,:)
               za_sed_c(:,:)  = 0.0
            endif
            if (jinorgben.eq.1) then
               zb_sed_si(:,:) = (atfp  * ( zb_sed_si(:,:) + za_sed_si(:,:) )) + (atfp1 * zn_sed_si(:,:))
               zn_sed_si(:,:) = za_sed_si(:,:)
               za_sed_si(:,:) = 0.0
               !!
               zb_sed_ca(:,:) = (atfp  * ( zb_sed_ca(:,:) + za_sed_ca(:,:) )) + (atfp1 * zn_sed_ca(:,:))
               zn_sed_ca(:,:) = za_sed_ca(:,:)
               za_sed_ca(:,:) = 0.0
            endif
         ENDIF
      ELSE                                                   !  case of smolar scheme or muscl
         if (jorgben.eq.1) then
            zb_sed_n(:,:)  = za_sed_n(:,:)
            zn_sed_n(:,:)  = za_sed_n(:,:)
            za_sed_n(:,:)  = 0.0
            !!
            zb_sed_fe(:,:) = za_sed_fe(:,:)
            zn_sed_fe(:,:) = za_sed_fe(:,:)
            za_sed_fe(:,:) = 0.0
            !!
            zb_sed_c(:,:)  = za_sed_c(:,:)
            zn_sed_c(:,:)  = za_sed_c(:,:)
            za_sed_c(:,:)  = 0.0
         endif
         if (jinorgben.eq.1) then
            zb_sed_si(:,:) = za_sed_si(:,:)
            zn_sed_si(:,:) = za_sed_si(:,:)
            za_sed_si(:,:) = 0.0
            !!
            zb_sed_ca(:,:) = za_sed_ca(:,:)
            zn_sed_ca(:,:) = za_sed_ca(:,:)
            za_sed_ca(:,:) = 0.0
         endif
      ENDIF
      endif

#if defined key_trc_diaadd
      !!----------------------------------------------------------------------
      !! Output several accumulated diagnostics
      !!   - biomass-average phytoplankton limitation terms
      !!   - integrated tendency terms
      !!----------------------------------------------------------------------
      !! 
      DO jj = 2,jpjm1
         DO ji = 2,jpim1
            !! non-diatom phytoplankton limitations
            trc2d(ji,jj,25) = trc2d(ji,jj,25) / ftot_pn(ji,jj)
            trc2d(ji,jj,26) = trc2d(ji,jj,26) / ftot_pn(ji,jj)
            trc2d(ji,jj,27) = trc2d(ji,jj,27) / ftot_pn(ji,jj)
            !! diatom phytoplankton limitations
            trc2d(ji,jj,28) = trc2d(ji,jj,28) / ftot_pd(ji,jj)
            trc2d(ji,jj,29) = trc2d(ji,jj,29) / ftot_pd(ji,jj)
            trc2d(ji,jj,30) = trc2d(ji,jj,30) / ftot_pd(ji,jj)
            trc2d(ji,jj,31) = trc2d(ji,jj,31) / ftot_pd(ji,jj)
            trc2d(ji,jj,32) = trc2d(ji,jj,32) / ftot_pd(ji,jj)
            !! tendency terms
            trc2d(ji,jj,76) = fflx_n(ji,jj)
            trc2d(ji,jj,77) = fflx_si(ji,jj)
            trc2d(ji,jj,78) = fflx_fe(ji,jj)
            !! integrated phytoplankton biomass
            trc2d(ji,jj,79) = ftot_pn(ji,jj)
            trc2d(ji,jj,80) = ftot_pd(ji,jj)
#if defined key_roam
            !! the balance of nitrogen production/consumption
            trc2d(ji,jj,111) = fnit_prod(ji,jj)  !! integrated nitrogen production
            trc2d(ji,jj,112) = fnit_cons(ji,jj)  !! integrated nitrogen consumption
            !! the balance of carbon production/consumption
            trc2d(ji,jj,113) = fcar_prod(ji,jj)  !! integrated carbon production
            trc2d(ji,jj,114) = fcar_cons(ji,jj)  !! integrated carbon consumption
            !! the balance of oxygen production/consumption
            trc2d(ji,jj,115) = foxy_prod(ji,jj)  !! integrated oxygen production
            trc2d(ji,jj,116) = foxy_cons(ji,jj)  !! integrated oxygen consumption
            trc2d(ji,jj,117) = foxy_anox(ji,jj)  !! integrated unrealised oxygen consumption
#endif
         END DO
      END DO

      !!----------------------------------------------------------------------
      !! Don't know what this does; belongs to someone else ...
      !!----------------------------------------------------------------------
      !! 
      !! Lateral boundary conditions on trc2d
      DO jn=1,jp_medusa_2d
          CALL lbc_lnk(trc2d(:,:,jn),'T',1. )
      END DO 

      !! Lateral boundary conditions on trc3d
      DO jn=1,jp_medusa_3d
          CALL lbc_lnk(trc3d(:,:,1,jn),'T',1. )
      END DO 
#endif

#if defined key_iomput
      !!----------------------------------------------------------------------
      !! Add in XML diagnostics stuff
      !!----------------------------------------------------------------------
      !!
      !! ** 2D diagnostics
      zw2d = trc2d(:,:, 1)
      CALL iom_put("INVTN"    ,zw2d)
      !!
      zw2d = trc2d(:,:, 2)
      CALL iom_put("INVTSI"   ,zw2d)
      !!
      zw2d = trc2d(:,:, 3)
      CALL iom_put("INVTFE"   ,zw2d)
      !!
      zw2d = trc2d(:,:, 4)
      CALL iom_put("PRN"      ,zw2d)
      !!
      zw2d = trc2d(:,:, 5)
      CALL iom_put("MPN"      ,zw2d)
      !!
      zw2d = trc2d(:,:, 6)
      CALL iom_put("PRD"      ,zw2d)
      !!
      zw2d = trc2d(:,:, 7)
      CALL iom_put("MPD"      ,zw2d)
      !!
      !! zw2d = trc2d(:,:, 8)
      !! CALL iom_put("DSED"     ,zw2d)
      !!
      zw2d = trc2d(:,:, 9)
      CALL iom_put("OPAL"     ,zw2d)
      !!
      zw2d = trc2d(:,:,10)
      CALL iom_put("OPALDISS" ,zw2d)
      !!
      zw2d = trc2d(:,:,11)
      CALL iom_put("GMIPN"    ,zw2d)
      !!
      zw2d = trc2d(:,:,12)
      CALL iom_put("GMID"     ,zw2d)
      !!
      zw2d = trc2d(:,:,13)
      CALL iom_put("MZMI"     ,zw2d)
      !!
      zw2d = trc2d(:,:,14)
      CALL iom_put("GMEPN"    ,zw2d)
      !!
      zw2d = trc2d(:,:,15)
      CALL iom_put("GMEPD"    ,zw2d)
      !!
      zw2d = trc2d(:,:,16)
      CALL iom_put("GMEZMI"   ,zw2d)
      !!
      zw2d = trc2d(:,:,17)
      CALL iom_put("GMED"     ,zw2d)
      !!
      zw2d = trc2d(:,:,18)
      CALL iom_put("MZME"     ,zw2d)
      !!
      !! zw2d = trc2d(:,:,19)
      !! CALL iom_put("DEXP"     ,zw2d)
      !!
      zw2d = trc2d(:,:,20)
      CALL iom_put("DETN"     ,zw2d)
      !!
      zw2d = trc2d(:,:,21)
      CALL iom_put("MDET"     ,zw2d)
      !!
      zw2d = trc2d(:,:,22)
      CALL iom_put("AEOLIAN"  ,zw2d)
      !!
      zw2d = trc2d(:,:,23)
      CALL iom_put("BENTHIC"  ,zw2d)
      !!
      zw2d = trc2d(:,:,24)
      CALL iom_put("SCAVENGE" ,zw2d)
      !!
      zw2d = trc2d(:,:,25)
      CALL iom_put("PN_JLIM"  ,zw2d)
      !!
      zw2d = trc2d(:,:,26)
      CALL iom_put("PN_NLIM"  ,zw2d)
      !!
      zw2d = trc2d(:,:,27)
      CALL iom_put("PN_FELIM" ,zw2d)
      !!
      zw2d = trc2d(:,:,28)
      CALL iom_put("PD_JLIM"  ,zw2d)
      !!
      zw2d = trc2d(:,:,29)
      CALL iom_put("PD_NLIM"  ,zw2d)
      !!
      zw2d = trc2d(:,:,30)
      CALL iom_put("PD_FELIM" ,zw2d)
      !!
      zw2d = trc2d(:,:,31)
      CALL iom_put("PD_SILIM" ,zw2d)
      !!
      zw2d = trc2d(:,:,32)
      CALL iom_put("PDSILIM2" ,zw2d)
      !!
      zw2d = trc2d(:,:,33)
      CALL iom_put("SDT__100" ,zw2d)
      !!
      zw2d = trc2d(:,:,34)
      CALL iom_put("SDT__200" ,zw2d)
      !!
      zw2d = trc2d(:,:,35)
      CALL iom_put("SDT__500" ,zw2d)
      !!
      zw2d = trc2d(:,:,36)
      CALL iom_put("SDT_1000" ,zw2d)
      !!
      zw2d = trc2d(:,:,37)
      CALL iom_put("TOTREG_N" ,zw2d)
      !!
      zw2d = trc2d(:,:,38)
      CALL iom_put("TOTRG_SI" ,zw2d)
      !!
      zw2d = trc2d(:,:,39)
      CALL iom_put("REG__100" ,zw2d)
      !!
      zw2d = trc2d(:,:,40)
      CALL iom_put("REG__200" ,zw2d)
      !!
      zw2d = trc2d(:,:,41)
      CALL iom_put("REG__500" ,zw2d)
      !!
      zw2d = trc2d(:,:,42)
      CALL iom_put("REG_1000" ,zw2d)
      !!
      zw2d = trc2d(:,:,43)
      CALL iom_put("FASTN"    ,zw2d)
      !!
      zw2d = trc2d(:,:,44)
      CALL iom_put("FASTSI"   ,zw2d)
      !!
      zw2d = trc2d(:,:,45)
      CALL iom_put("FASTFE"   ,zw2d)
      !!
      zw2d = trc2d(:,:,46)
      CALL iom_put("FASTC"    ,zw2d)
      !!
      zw2d = trc2d(:,:,47)
      CALL iom_put("FASTCA"   ,zw2d)
      !!
      zw2d = trc2d(:,:,48)
      CALL iom_put("FDT__100" ,zw2d)
      !!
      zw2d = trc2d(:,:,49)
      CALL iom_put("FDT__200" ,zw2d)
      !!
      zw2d = trc2d(:,:,50)
      CALL iom_put("FDT__500" ,zw2d)
      !!
      zw2d = trc2d(:,:,51)
      CALL iom_put("FDT_1000" ,zw2d)
      !!
      zw2d = trc2d(:,:,52)
      CALL iom_put("RG__100F" ,zw2d)
      !!
      zw2d = trc2d(:,:,53)
      CALL iom_put("RG__200F" ,zw2d)
      !!
      zw2d = trc2d(:,:,54)
      CALL iom_put("RG__500F" ,zw2d)
      !!
      zw2d = trc2d(:,:,55)
      CALL iom_put("RG_1000F" ,zw2d)
      !!
      zw2d = trc2d(:,:,56)
      CALL iom_put("FDS__100" ,zw2d)
      !!
      zw2d = trc2d(:,:,57)
      CALL iom_put("FDS__200" ,zw2d)
      !!
      zw2d = trc2d(:,:,58)
      CALL iom_put("FDS__500" ,zw2d)
      !!
      zw2d = trc2d(:,:,59)
      CALL iom_put("FDS_1000" ,zw2d)
      !!
      zw2d = trc2d(:,:,60)
      CALL iom_put("RGS__100" ,zw2d)
      !!
      zw2d = trc2d(:,:,61)
      CALL iom_put("RGS__200" ,zw2d)
      !!
      zw2d = trc2d(:,:,62)
      CALL iom_put("RGS__500" ,zw2d)
      !!
      zw2d = trc2d(:,:,63)
      CALL iom_put("RGS_1000" ,zw2d)
      !!
      zw2d = trc2d(:,:,64)
      CALL iom_put("REMINN"   ,zw2d)
      !!
      zw2d = trc2d(:,:,65)
      CALL iom_put("REMINSI"  ,zw2d)
      !!
      zw2d = trc2d(:,:,66)
      CALL iom_put("REMINFE"  ,zw2d)
      !!
      zw2d = trc2d(:,:,67)
      CALL iom_put("REMINC"   ,zw2d)
      !!
      zw2d = trc2d(:,:,68)
      CALL iom_put("REMINCA"  ,zw2d)
      !!
      zw2d = trc2d(:,:,69)
      CALL iom_put("SEAFLRN"  ,zw2d)
      !!
      zw2d = trc2d(:,:,70)
      CALL iom_put("SEAFLRSI" ,zw2d)
      !!
      zw2d = trc2d(:,:,71)
      CALL iom_put("SEAFLRFE" ,zw2d)
      !!
      zw2d = trc2d(:,:,72)
      CALL iom_put("SEAFLRC"  ,zw2d)
      !!
      zw2d = trc2d(:,:,73)
      CALL iom_put("SEAFLRCA" ,zw2d)
      !!
      zw2d = trc2d(:,:,74)
      CALL iom_put("MED_QSR"  ,zw2d)
      !!
      zw2d = trc2d(:,:,75)
      CALL iom_put("MED_XPAR" ,zw2d)
      !!
      zw2d = trc2d(:,:,76)
      CALL iom_put("INTFLX_N" ,zw2d)
      !!
      zw2d = trc2d(:,:,77)
      CALL iom_put("INTFX_SI" ,zw2d)
      !!
      zw2d = trc2d(:,:,78)
      CALL iom_put("INTFX_FE" ,zw2d)
      !!
      zw2d = trc2d(:,:,79)
      CALL iom_put("INT_PN"   ,zw2d)
      !!
      zw2d = trc2d(:,:,80)
      CALL iom_put("INT_PD"   ,zw2d)
      !!
      zw2d = trc2d(:,:,81)
      CALL iom_put("ML_PRN"   ,zw2d)
      !!
      zw2d = trc2d(:,:,82)
      CALL iom_put("ML_PRD"   ,zw2d)
      !!
      zw2d = trc2d(:,:,83)
      CALL iom_put("OCAL_CCD" ,zw2d)
      !!
      zw2d = trc2d(:,:,84)
      CALL iom_put("OCAL_LVL" ,zw2d)
      !!
      zw2d = trc2d(:,:,85)
      CALL iom_put("FE_0000"  ,zw2d)
      !!
      zw2d = trc2d(:,:,86)
      CALL iom_put("FE_0100"  ,zw2d)
      !!
      zw2d = trc2d(:,:,87)
      CALL iom_put("FE_0200"  ,zw2d)
      !!
      zw2d = trc2d(:,:,88)
      CALL iom_put("FE_0500"  ,zw2d)
      !!
      zw2d = trc2d(:,:,89)
      CALL iom_put("FE_1000"  ,zw2d)
      !!
      zw2d = trc2d(:,:,90)
      CALL iom_put("MED_XZE"  ,zw2d)
# if defined key_roam
      !!
      zw2d = trc2d(:,:,91)
      CALL iom_put("WIND"     ,zw2d)
      !!
      zw2d = trc2d(:,:,92)
      CALL iom_put("ATM_PCO2" ,zw2d)
      !!
      zw2d = trc2d(:,:,93)
      CALL iom_put("OCN_PH"   ,zw2d)
      !!
      zw2d = trc2d(:,:,94)
      CALL iom_put("OCN_PCO2" ,zw2d)
      !!
      zw2d = trc2d(:,:,95)
      CALL iom_put("OCNH2CO3" ,zw2d)
      !!
      zw2d = trc2d(:,:,96)
      CALL iom_put("OCN_HCO3" ,zw2d)
      !!
      zw2d = trc2d(:,:,97)
      CALL iom_put("OCN_CO3"  ,zw2d)
      !!
      zw2d = trc2d(:,:,98)
      CALL iom_put("CO2FLUX"  ,zw2d)
      !!
      zw2d = trc2d(:,:,99)
      CALL iom_put("OM_CAL"   ,zw2d)
      !!
      zw2d = trc2d(:,:,100)
      CALL iom_put("OM_ARG"   ,zw2d)
      !!
      zw2d = trc2d(:,:,101)
      CALL iom_put("TCO2"     ,zw2d)
      !!
      zw2d = trc2d(:,:,102)
      CALL iom_put("TALK"     ,zw2d)
      !!
      zw2d = trc2d(:,:,103)
      CALL iom_put("KW660"    ,zw2d)
      !!
      zw2d = trc2d(:,:,104)
      CALL iom_put("ATM_PP0"  ,zw2d)
      !!
      zw2d = trc2d(:,:,105)
      CALL iom_put("O2FLUX"   ,zw2d)
      !!
      zw2d = trc2d(:,:,106)
      CALL iom_put("O2SAT"    ,zw2d)
      !!
      zw2d = trc2d(:,:,107)
      CALL iom_put("CAL_CCD"  ,zw2d)
      !!
      zw2d = trc2d(:,:,108)
      CALL iom_put("ARG_CCD"  ,zw2d)
      !!
      zw2d = trc2d(:,:,109)
      CALL iom_put("SFR_OCAL" ,zw2d)
      !!
      zw2d = trc2d(:,:,110)
      CALL iom_put("SFR_OARG" ,zw2d)
      !!
      zw2d = trc2d(:,:,111)
      CALL iom_put("N_PROD"   ,zw2d)
      !!
      zw2d = trc2d(:,:,112)
      CALL iom_put("N_CONS"   ,zw2d)
      !!
      zw2d = trc2d(:,:,113)
      CALL iom_put("C_PROD"   ,zw2d)
      !!
      zw2d = trc2d(:,:,114)
      CALL iom_put("C_CONS"   ,zw2d)
      !!
      zw2d = trc2d(:,:,115)
      CALL iom_put("O2_PROD"  ,zw2d)
      !!
      zw2d = trc2d(:,:,116)
      CALL iom_put("O2_CONS"  ,zw2d)
      !!
      zw2d = trc2d(:,:,117)
      CALL iom_put("O2_ANOX"  ,zw2d)
      !!
      zw2d = trc2d(:,:,118)
      CALL iom_put("RR_0100"  ,zw2d)
      !!
      zw2d = trc2d(:,:,119)
      CALL iom_put("RR_0500"  ,zw2d)
      !!
      zw2d = trc2d(:,:,120)
      CALL iom_put("RR_1000"  ,zw2d)
      !!
      zw2d = trc2d(:,:,121)
      CALL iom_put("IBEN_N"   ,zw2d)
      !!
      zw2d = trc2d(:,:,122)
      CALL iom_put("IBEN_FE"  ,zw2d)
      !!
      zw2d = trc2d(:,:,123)
      CALL iom_put("IBEN_C"   ,zw2d)
      !!
      zw2d = trc2d(:,:,124)
      CALL iom_put("IBEN_SI"  ,zw2d)
      !!
      zw2d = trc2d(:,:,125)
      CALL iom_put("IBEN_CA"  ,zw2d)
      !!
      zw2d = trc2d(:,:,126)
      CALL iom_put("OBEN_N"   ,zw2d)
      !!
      zw2d = trc2d(:,:,127)
      CALL iom_put("OBEN_FE"  ,zw2d)
      !!
      zw2d = trc2d(:,:,128)
      CALL iom_put("OBEN_C"   ,zw2d)
      !!
      zw2d = trc2d(:,:,129)
      CALL iom_put("OBEN_SI"  ,zw2d)
      !!
      zw2d = trc2d(:,:,130)
      CALL iom_put("OBEN_CA"  ,zw2d)
      !!
      zw2d = trc2d(:,:,131)
      CALL iom_put("BEN_N"    ,zw2d)
      !!
      zw2d = trc2d(:,:,132)
      CALL iom_put("BEN_FE"   ,zw2d)
      !!
      zw2d = trc2d(:,:,133)
      CALL iom_put("BEN_C"    ,zw2d)
      !!
      zw2d = trc2d(:,:,134)
      CALL iom_put("BEN_SI"   ,zw2d)
      !!
      zw2d = trc2d(:,:,135)
      CALL iom_put("BEN_CA"   ,zw2d)
      !!
      zw2d = trc2d(:,:,136)
      CALL iom_put("RUNOFF"   ,zw2d)
      !!
      zw2d = trc2d(:,:,137)
      CALL iom_put("RIV_N"    ,zw2d)
      !!
      zw2d = trc2d(:,:,138)
      CALL iom_put("RIV_SI"   ,zw2d)
      !!
      zw2d = trc2d(:,:,139)
      CALL iom_put("RIV_C"    ,zw2d)
      !!
      zw2d = trc2d(:,:,140)
      CALL iom_put("RIV_ALK"  ,zw2d)
      !!
      zw2d = trc2d(:,:,141)
      CALL iom_put("DETC"     ,zw2d)
      !!
      zw2d = trc2d(:,:,142)
      CALL iom_put("SDC__100" ,zw2d)
      !!
      zw2d = trc2d(:,:,143)
      CALL iom_put("SDC__200" ,zw2d)
      !!
      zw2d = trc2d(:,:,144)
      CALL iom_put("SDC__500" ,zw2d)
      !!
      zw2d = trc2d(:,:,145)
      CALL iom_put("SDC_1000" ,zw2d)
      !!
      zw2d = trc2d(:,:,146)
      CALL iom_put("INVTC"    ,zw2d)
      !!
      zw2d = trc2d(:,:,147)
      CALL iom_put("INVTALK"  ,zw2d)
      !!
      zw2d = trc2d(:,:,148)
      CALL iom_put("INVTO2"   ,zw2d)
      !!
      zw2d = trc2d(:,:,149)
      CALL iom_put("LYSO_CA"  ,zw2d)
      !!
      zw2d = trc2d(:,:,150)
      CALL iom_put("BLANK_1"  ,zw2d)
      !!
# endif
      !!
# if defined key_roam
      !! ** 3D diagnostics
      zw3d = trc3d(:,:,:, 1)
      CALL iom_put("TPP3"     ,zw3d)
      !!
      zw3d = trc3d(:,:,:, 2)
      CALL iom_put("DETFLUX3" ,zw3d)
      !!
      zw3d = trc3d(:,:,:, 3)
      CALL iom_put("REMIN3N"  ,zw3d)
      !!
      zw3d = trc3d(:,:,:, 4)
      CALL iom_put("PH3"      ,zw3d)
      !!
      zw3d = trc3d(:,:,:, 5)
      CALL iom_put("OM_CAL3"  ,zw3d)
# else
      !! ** 3D diagnostics
      zw3d = trc3d(:,:,:, 1)
      CALL iom_put("TPP3"     ,zw3d)
      !!
      zw3d = trc3d(:,:,:, 2)
      CALL iom_put("DETFLUX3" ,zw3d)
      !!
      zw3d = trc3d(:,:,:, 3)
      CALL iom_put("REMIN3N"  ,zw3d)
      !!
      zw3d = trc3d(:,:,:, 4)
      CALL iom_put("REMIN3SI" ,zw3d)
# endif
#endif

#if defined key_trc_diabio
      !! Lateral boundary conditions on trcbio
      DO jn=1,jp_medusa_trd
          CALL lbc_lnk(trbio(:,:,1,jn),'T',1. )
      END DO 
#endif

   END SUBROUTINE trc_bio_medusa

#  else
   !!======================================================================
   !!  Dummy module :                                   No MEDUSA bio-model
   !!======================================================================
CONTAINS
   SUBROUTINE trc_bio_medusa( kt )                   ! Empty routine
      INTEGER, INTENT( in ) ::   kt
      WRITE(*,*) 'trc_bio_medusa: You should not have seen this print! error?', kt
   END SUBROUTINE trc_bio_medusa
#endif 

   !!======================================================================
END MODULE  trcbio_medusa
